@@ -21,8 +21,8 @@
 #include <QDesktopServices>
 #include <QEvent>
 #include <QGuiApplication>
+#include <QStyleHints>
 #include <QPointer>
-#include <QScreen>
 #include <QString>
 #include <QThread>
 #include <QTouchDevice>
@@ -30,6 +30,7 @@
 #include <QtGui/qpa/qplatformnativeinterface.h>
 
 #include "base/lazy_instance.h"
+#include "base/memory/ptr_util.h"
 #include "base/single_thread_task_runner.h"
 #include "base/thread_task_runner_handle.h"
 #include "content/public/browser/browser_thread.h"
@@ -41,9 +42,11 @@
 #include "oxide_qt_browser_startup.h"
 #include "oxide_qt_browser_thread_q_event_dispatcher.h"
 #include "oxide_qt_clipboard.h"
+#include "oxide_qt_drag_source.h"
 #include "oxide_qt_location_provider.h"
 #include "oxide_qt_message_pump.h"
-#include "oxide_qt_screen_utils.h"
+#include "oxide_qt_screen_client.h"
+#include "oxide_qt_vibration_manager.h"
 
 namespace oxide {
 namespace qt {
@@ -85,16 +88,6 @@ void BrowserPlatformIntegration::OnClipboardDataChanged() {
   NotifyClipboardDataChanged();
 }
 
-void BrowserPlatformIntegration::OnScreenGeometryChanged(
-    const QRect& geometry) {
-  UpdateDefaultScreenInfo();
-}
-
-void BrowserPlatformIntegration::OnScreenOrientationChanged(
-    Qt::ScreenOrientation orientation) {
-  UpdateDefaultScreenInfo();
-}
-
 void BrowserPlatformIntegration::UpdateApplicationState() {
   ApplicationState state = CalculateApplicationState(suspended_);
   if (state == state_) {
@@ -106,12 +99,6 @@ void BrowserPlatformIntegration::UpdateApplicationState() {
   NotifyApplicationStateChanged();
 }
 
-void BrowserPlatformIntegration::UpdateDefaultScreenInfo() {
-  base::AutoLock lock(default_screen_info_lock_);
-  default_screen_info_ =
-      GetWebScreenInfoFromQScreen(QGuiApplication::primaryScreen());
-}
-
 bool BrowserPlatformIntegration::LaunchURLExternally(const GURL& url) {
   if (!content::BrowserThread::CurrentlyOn(content::BrowserThread::UI)) {
     content::BrowserThread::PostTask(
@@ -144,9 +131,8 @@ intptr_t BrowserPlatformIntegration::GetNativeDisplay() {
                                    QGuiApplication::primaryScreen()));
 }
 
-blink::WebScreenInfo BrowserPlatformIntegration::GetDefaultScreenInfo() {
-  base::AutoLock lock(default_screen_info_lock_);
-  return default_screen_info_;
+oxide::ScreenClient* BrowserPlatformIntegration::GetScreenClient() {
+  return screen_client_.get();
 }
 
 oxide::GLContextDependent* BrowserPlatformIntegration::GetGLShareContext() {
@@ -154,9 +140,9 @@ oxide::GLContextDependent* BrowserPlatformIntegration::GetGLShareContext() {
   return BrowserStartup::GetInstance()->shared_gl_context();
 }
 
-scoped_ptr<oxide::MessagePump>
+std::unique_ptr<oxide::MessagePump>
 BrowserPlatformIntegration::CreateUIMessagePump() {
-  return make_scoped_ptr(new MessagePump());
+  return base::WrapUnique(new MessagePump());
 }
 
 ui::Clipboard* BrowserPlatformIntegration::CreateClipboard() {
@@ -175,7 +161,7 @@ void BrowserPlatformIntegration::BrowserThreadInit(
   g_io_thread.Get() = thread;
 }
 
-scoped_ptr<content::LocationProvider>
+std::unique_ptr<content::LocationProvider>
 BrowserPlatformIntegration::CreateLocationProvider() {
   // Give the geolocation thread a Qt event dispatcher, so that we can use
   // Queued signals / slots between it and the IO thread
@@ -185,7 +171,7 @@ BrowserPlatformIntegration::CreateLocationProvider() {
       new BrowserThreadQEventDispatcher(base::ThreadTaskRunnerHandle::Get()));
   }
 
-  return make_scoped_ptr(new LocationProvider());
+  return base::WrapUnique(new LocationProvider());
 }
 
 oxide::BrowserPlatformIntegration::ApplicationState
@@ -194,10 +180,19 @@ BrowserPlatformIntegration::GetApplicationState() {
   return state_;
 }
 
+int BrowserPlatformIntegration::GetClickInterval() {
+  return qApp->styleHints()->mouseDoubleClickInterval();
+}
+
 std::string BrowserPlatformIntegration::GetApplicationName() {
   return application_name_;
 }
 
+std::unique_ptr<oxide::DragSource> BrowserPlatformIntegration::CreateDragSource(
+    oxide::DragSourceClient* client) {
+  return base::WrapUnique(new DragSource(client));
+}
+
 bool BrowserPlatformIntegration::eventFilter(QObject* watched, QEvent* event) {
   if (event->type() == QEvent::ApplicationActivate ||
       event->type() == QEvent::ApplicationDeactivate) {
@@ -211,24 +206,8 @@ bool BrowserPlatformIntegration::eventFilter(QObject* watched, QEvent* event) {
 BrowserPlatformIntegration::BrowserPlatformIntegration()
     : application_name_(qApp->applicationName().toStdString()),
       suspended_(false),
-      state_(CalculateApplicationState(false)) {
-  QScreen* primary_screen = QGuiApplication::primaryScreen();
-  primary_screen->setOrientationUpdateMask(Qt::LandscapeOrientation |
-                                           Qt::PortraitOrientation |
-                                           Qt::InvertedLandscapeOrientation |
-                                           Qt::InvertedPortraitOrientation);
-  connect(primary_screen, SIGNAL(virtualGeometryChanged(const QRect&)),
-          SLOT(OnScreenGeometryChanged(const QRect&)));
-  connect(primary_screen, SIGNAL(geometryChanged(const QRect&)),
-          SLOT(OnScreenGeometryChanged(const QRect&)));
-  connect(primary_screen, SIGNAL(orientationChanged(Qt::ScreenOrientation)),
-          SLOT(OnScreenOrientationChanged(Qt::ScreenOrientation)));
-  connect(primary_screen,
-          SIGNAL(primaryOrientationChanged(Qt::ScreenOrientation)),
-          SLOT(OnScreenOrientationChanged(Qt::ScreenOrientation)));
-
-  UpdateDefaultScreenInfo();
-
+      state_(CalculateApplicationState(false)),
+      screen_client_(new ScreenClient()) {
   connect(qApp, SIGNAL(applicationStateChanged(Qt::ApplicationState)),
           SLOT(OnApplicationStateChanged()));
   connect(QGuiApplication::clipboard(), SIGNAL(dataChanged()),
@@ -242,6 +221,11 @@ BrowserPlatformIntegration::~BrowserPlatformIntegration() {
   qApp->removeEventFilter(this);
 }
 
+void BrowserPlatformIntegration::CreateVibrationManager(
+      mojo::InterfaceRequest<device::VibrationManager> request) {
+  VibrationManager::Create(std::move(request));
+}
+
 QThread* GetIOQThread() {
   return g_io_thread.Get();
 }
