@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2014-2015 Canonical Ltd.
+// Copyright (C) 2014-2016 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -17,6 +17,8 @@
 
 #include "oxide_qt_browser_startup.h"
 
+#include <queue>
+
 #include <QCoreApplication>
 #include <QGlobalStatic>
 #include <QGuiApplication>
@@ -26,13 +28,17 @@
 #include <QtGui/private/qopenglcontext_p.h>
 #endif
 
+#include "base/lazy_instance.h"
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "ui/gfx/geometry/size.h"
 
 #include "qt/core/app/oxide_qt_platform_delegate.h"
 #include "qt/core/gpu/oxide_qt_gl_context_dependent.h"
 
+#include "oxide_qt_dpi_utils.h"
 #include "oxide_qt_screen_utils.h"
+#include "oxide_qt_type_conversions.h"
 #include "oxide_qt_web_context.h"
 
 using oxide::BrowserProcessMain;
@@ -44,16 +50,24 @@ namespace {
 
 Q_GLOBAL_STATIC(BrowserStartup, g_instance)
 
+base::LazyInstance<std::queue<base::Closure>> g_shutdown_callbacks =
+    LAZY_INSTANCE_INITIALIZER;
+
 void ShutdownChromium() {
-  WebContext::DestroyDefault();
+  std::queue<base::Closure>& callbacks = g_shutdown_callbacks.Get();
+  while (!callbacks.empty()) {
+    base::Closure callback = callbacks.front();
+    callbacks.pop();
+    callback.Run();
+  }
+
   BrowserProcessMain::GetInstance()->Shutdown();
 }
 
 }
 
 BrowserStartup::BrowserStartup()
-    : process_model_is_from_env_(false),
-      process_model_(oxide::PROCESS_MODEL_UNDEFINED) {}
+    : process_model_(oxide::PROCESS_MODEL_UNDEFINED) {}
 
 // static
 BrowserStartup* BrowserStartup::GetInstance() {
@@ -89,8 +103,6 @@ oxide::ProcessModel BrowserStartup::GetProcessModel() {
         oxide::BrowserProcessMain::GetProcessModelOverrideFromEnv();
     if (process_model_ == oxide::PROCESS_MODEL_UNDEFINED) {
       process_model_ = oxide::PROCESS_MODEL_MULTI_PROCESS;
-    } else {
-      process_model_is_from_env_ = true;
     }
   }
 
@@ -105,18 +117,6 @@ void BrowserStartup::SetProcessModel(oxide::ProcessModel model) {
 
   DCHECK_NE(model, oxide::PROCESS_MODEL_UNDEFINED);
 
-  static bool did_warn = false;
-  if (!did_warn) {
-    did_warn = true;
-    qWarning()
-        << "Changing the Oxide process model is currently experimental "
-        << "and untested, and might break your application in unexpected "
-        << "ways. This interface is only exposed for development purposes "
-        << "at the moment, although it will eventually be suitable for "
-        << "public use";
-  }
-
-  process_model_is_from_env_ = false;
   process_model_ = model;
 }
 
@@ -127,10 +127,6 @@ void BrowserStartup::SetSharedGLContext(GLContextDependent* context) {
 }
 #endif
 
-bool BrowserStartup::DidSelectProcessModelFromEnv() const {
-  return process_model_is_from_env_;
-}
-
 void BrowserStartup::EnsureChromiumStarted() {
   if (BrowserProcessMain::GetInstance()->IsRunning()) {
     return;
@@ -148,7 +144,7 @@ void BrowserStartup::EnsureChromiumStarted() {
 #endif
 
   BrowserProcessMain::StartParams params(
-      make_scoped_ptr(new PlatformDelegate()));
+      base::WrapUnique(new PlatformDelegate()));
 #if defined(USE_NSS_CERTS)
   params.nss_db_path = GetNSSDbPath();
 #endif
@@ -180,16 +176,19 @@ void BrowserStartup::EnsureChromiumStarted() {
   }
 
   QScreen* primary_screen = QGuiApplication::primaryScreen();
-  QSize primary_screen_size = primary_screen->size();
-  params.primary_screen_size_dip =
-      gfx::ScaleToRoundedSize(
-          gfx::Size(primary_screen_size.width(), primary_screen_size.height()),
-          1 / GetDeviceScaleFactorFromQScreen(primary_screen));
+  params.primary_screen_size =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(primary_screen->size()),
+                                          primary_screen);
 
   oxide::BrowserProcessMain::GetInstance()->Start(std::move(params));
 
   qAddPostRoutine(ShutdownChromium);
 }
 
+// static
+void BrowserStartup::AddShutdownCallback(const base::Closure& callback) {
+  g_shutdown_callbacks.Get().push(callback);
+}
+
 } // namespace qt
 } // namespace oxide
