@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2014 Canonical Ltd.
+// Copyright (C) 2014-2016 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -24,17 +24,18 @@
 #include <QMouseEvent>
 #include <QPoint>
 #include <Qt>
-#include <QTouchEvent>
 #include <QWheelEvent>
 
 #include "base/logging.h"
 #include "base/time/time.h"
 #include "ui/events/keycodes/keyboard_codes.h"
 #include "ui/gfx/geometry/point_f.h"
-#include "ui/gfx/geometry/vector2d_f.h"
 
 #include "shared/browser/oxide_event_utils.h"
 
+#include "oxide_qt_dpi_utils.h"
+#include "oxide_qt_type_conversions.h"
+
 namespace oxide {
 namespace qt {
 
@@ -85,7 +86,13 @@ blink::WebUChar GetControlCharacter(int key_code, bool shift) {
 }
 
 double QInputEventTimeToWebEventTime(QInputEvent* qevent) {
-  return static_cast<double>(qevent->timestamp() / 1000.0);
+  if (getenv("OXIDE_TESTING_MODE")) {
+    // We don't have timestamps in testing
+    return base::TimeDelta(base::TimeTicks::Now() - base::TimeTicks())
+        .InSecondsF();
+  }
+
+  return double(qevent->timestamp()) / 1000;
 }
 
 int QInputEventStateToWebEventModifiers(QInputEvent* qevent) {
@@ -406,7 +413,7 @@ int QKeyEventKeyCodeToWebEventKeyCode(QKeyEvent* qevent) {
   default:
     break;
   }
-  
+
   return 0;
 }
 
@@ -430,78 +437,6 @@ int QMouseEventStateToWebEventModifiers(QMouseEvent* qevent) {
   return modifiers;
 }
 
-ui::EventType QTouchPointStateToEventType(Qt::TouchPointState state) {
-  switch (state) {
-    case Qt::TouchPointPressed:
-      return ui::ET_TOUCH_PRESSED;
-    case Qt::TouchPointMoved:
-      return ui::ET_TOUCH_MOVED;
-    case Qt::TouchPointReleased:
-      return ui::ET_TOUCH_RELEASED;
-    default:
-      NOTREACHED();
-      return ui::ET_UNKNOWN;
-  }
-}
-
-}
-
-UITouchEventFactory::UITouchEventFactory() {}
-
-UITouchEventFactory::~UITouchEventFactory() {}
-
-void UITouchEventFactory::MakeEvents(QTouchEvent* event,
-                                     float device_scale,
-                                     float location_bar_content_offset_dip,
-                                     ScopedVector<ui::TouchEvent>* results) {
-  // The event’s timestamp is not guaranteed to have the same origin as the
-  // internal timedelta used by chromium to calculate speed and displacement
-  // for a fling gesture, so we can’t use it.
-  base::TimeDelta timestamp(base::TimeTicks::Now() - base::TimeTicks());
-
-  float scale = 1 / device_scale;
-
-  for (int i = 0; i < event->touchPoints().size(); ++i) {
-    const QTouchEvent::TouchPoint& touch_point = event->touchPoints().at(i);
-
-    if (touch_point.state() == Qt::TouchPointStationary) {
-      continue;
-    }
-
-    if (touch_point.state() == Qt::TouchPointPressed) {
-      touch_point_content_offsets_[touch_point.id()] =
-          location_bar_content_offset_dip;
-    }
-
-    gfx::PointF location =
-        gfx::ScalePoint(gfx::PointF(touch_point.pos().x(),
-                                    touch_point.pos().y()),
-                        scale);
-    location +=
-        gfx::Vector2dF(0, -touch_point_content_offsets_[touch_point.id()]);
-
-    ui::TouchEvent* ui_event = new ui::TouchEvent(
-        QTouchPointStateToEventType(touch_point.state()),
-        gfx::Point(location.x(), location.y()),
-        0,
-        touch_point.id(),
-        timestamp,
-        0.0f, 0.0f,
-        0.0f,
-        float(touch_point.pressure()));
-    gfx::PointF root_location =
-        gfx::ScalePoint(gfx::PointF(touch_point.screenPos().x(),
-                                    touch_point.screenPos().y()),
-                        scale);
-    ui_event->set_root_location(
-        gfx::Point(root_location.x(), root_location.y()));
-
-    results->push_back(ui_event);
-
-    if (touch_point.state() == Qt::TouchPointReleased) {
-      touch_point_content_offsets_.erase(touch_point.id());
-    }
-  }
 }
 
 content::NativeWebKeyboardEvent MakeNativeWebKeyboardEvent(QKeyEvent* event,
@@ -562,22 +497,29 @@ content::NativeWebKeyboardEvent MakeNativeWebKeyboardEvent(QKeyEvent* event,
 }
 
 blink::WebMouseEvent MakeWebMouseEvent(QMouseEvent* event,
-                                       float device_scale,
-                                       float location_bar_content_offset_dip) {
+                                       QScreen* screen,
+                                       float location_bar_content_offset) {
   blink::WebMouseEvent result;
 
   result.timeStampSeconds = QInputEventTimeToWebEventTime(event);
   result.modifiers = QMouseEventStateToWebEventModifiers(event);
 
-  result.x = qRound(event->x() / device_scale);
-  result.y = qRound((event->y() / device_scale) -
-                    location_bar_content_offset_dip);
+  gfx::Point pos =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(event->pos()), screen);
+  result.x = pos.x();
+  result.y = std::floor(pos.y() - location_bar_content_offset);
 
-  result.windowX = qRound(event->windowPos().x() / device_scale);
-  result.windowY = qRound(event->windowPos().y() / device_scale);
+  gfx::PointF window_pos =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(event->windowPos()),
+                                          screen);
+  result.windowX = std::floor(window_pos.x());
+  result.windowY = std::floor(window_pos.y());
 
-  result.globalX = qRound(event->globalX() / device_scale);
-  result.globalY = qRound(event->globalY() / device_scale);
+  gfx::Point global_pos =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(event->globalPos()),
+                                          screen);
+  result.globalX = global_pos.x();
+  result.globalY = global_pos.y();
 
   result.clickCount = 0;
 
@@ -592,10 +534,6 @@ blink::WebMouseEvent MakeWebMouseEvent(QMouseEvent* event,
     case QEvent::MouseMove:
       result.type = blink::WebInputEvent::MouseMove;
       break;
-    case QEvent::MouseButtonDblClick:
-      result.type = blink::WebInputEvent::MouseDown;
-      result.clickCount = 2;
-      break;
     default:
       NOTREACHED();
   }
@@ -630,9 +568,9 @@ blink::WebMouseEvent MakeWebMouseEvent(QMouseEvent* event,
 }
 
 blink::WebMouseWheelEvent MakeWebMouseWheelEvent(QWheelEvent* event,
-                                                 const QPoint& window_pos,
-                                                 float device_scale,
-                                                 float location_bar_content_offset_dip) {
+                                                 const QPointF& window_pos,
+                                                 QScreen* screen,
+                                                 float location_bar_content_offset) {
   blink::WebMouseWheelEvent result;
 
   result.timeStampSeconds = QInputEventTimeToWebEventTime(event);
@@ -651,14 +589,21 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEvent(QWheelEvent* event,
   result.type = blink::WebInputEvent::MouseWheel;
   result.button = blink::WebMouseEvent::ButtonNone;
 
-  result.x = qRound(event->x() / device_scale);
-  result.y = qRound((event->y() / device_scale) - location_bar_content_offset_dip);
+  gfx::Point pos =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(event->pos()), screen);
+  result.x = pos.x();
+  result.y = std::floor(pos.y() - location_bar_content_offset);
 
-  result.windowX = qRound(window_pos.x() / device_scale);
-  result.windowY = qRound(window_pos.y() / device_scale);
+  gfx::PointF converted_window_pos =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(window_pos), screen);
+  result.windowX = std::floor(converted_window_pos.x());
+  result.windowY = std::floor(converted_window_pos.y());
 
-  result.globalX = qRound(event->globalX() / device_scale);
-  result.globalY = qRound(event->globalY() / device_scale);
+  gfx::Point global_pos =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(event->globalPos()),
+                                          screen);
+  result.globalX = global_pos.x();
+  result.globalY = global_pos.y();
 
   // See comment in third_party/WebKit/Source/web/gtk/WebInputEventFactory.cpp
   static const float scrollbarPixelsPerTick = 160.0f / 3.0f;
@@ -676,24 +621,29 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEvent(QWheelEvent* event,
 
 blink::WebMouseEvent MakeWebMouseEvent(
     QHoverEvent* event,
-    const QPoint& window_pos,
+    const QPointF& window_pos,
     const QPoint& global_pos,
-    float device_scale,
-    float location_bar_content_offset_dip) {
+    QScreen* screen,
+    float location_bar_content_offset) {
   blink::WebMouseEvent result;
 
   result.timeStampSeconds = QInputEventTimeToWebEventTime(event);
   result.modifiers = QInputEventStateToWebEventModifiers(event);
 
-  result.x = qRound(event->pos().x() / device_scale);
-  result.y = qRound((event->pos().y() / device_scale) -
-                    location_bar_content_offset_dip);
+  gfx::Point pos =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(event->pos()), screen);
+  result.x = pos.x();
+  result.y = std::floor(pos.y() - location_bar_content_offset);
 
-  result.windowX = qRound(window_pos.x() / device_scale);
-  result.windowY = qRound(window_pos.x() / device_scale);
+  gfx::PointF converted_window_pos =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(window_pos), screen);
+  result.windowX = std::floor(converted_window_pos.x());
+  result.windowY = std::floor(converted_window_pos.y());
 
-  result.globalX = qRound(global_pos.x() / device_scale);
-  result.globalY = qRound(global_pos.y() / device_scale);
+  gfx::Point converted_global_pos =
+      DpiUtils::ConvertQtPixelsToChromium(ToChromium(global_pos), screen);
+  result.globalX = converted_global_pos.x();
+  result.globalY = converted_global_pos.y();
 
   result.clickCount = 0;
 
