@@ -1,5 +1,5 @@
 // vim:expandtab:shiftwidth=2:tabstop=2:
-// Copyright (C) 2013-2015 Canonical Ltd.
+// Copyright (C) 2013-2016 Canonical Ltd.
 
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Lesser General Public
@@ -33,6 +33,7 @@
 #include "base/logging.h"
 #include "base/strings/string_util.h"
 #include "base/synchronization/lock.h"
+#include "base/thread_task_runner_handle.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/cookie_store_factory.h"
 #include "content/public/browser/resource_request_info.h"
@@ -40,7 +41,8 @@
 #include "net/base/net_errors.h"
 #include "net/base/static_cookie_policy.h"
 #include "net/cookies/canonical_cookie.h"
-#include "net/cookies/cookie_monster.h"
+#include "net/cookies/cookie_constants.h"
+#include "net/cookies/cookie_store.h"
 #include "net/http/http_response_headers.h"
 #include "net/url_request/url_request.h"
 #include "url/gurl.h"
@@ -73,10 +75,6 @@ namespace {
 
 const unsigned kDefaultDevtoolsPort = 8484;
 
-bool g_default_is_application_owned;
-bool g_initialized_default = false;
-WebContext* g_default;
-
 int GetNextCookieRequestId() {
   static int id = 0;
   if (id == std::numeric_limits<int>::max()) {
@@ -164,10 +162,11 @@ struct WebContext::ConstructProperties {
 
 class SetCookiesContext : public base::RefCounted<SetCookiesContext> {
  public:
-  SetCookiesContext(int id)
-      : id(id), remaining(0) {}
+  SetCookiesContext(int request_id)
+      : request_id(request_id),
+        remaining(0) {}
 
-  int id;
+  int request_id;
   int remaining;
   QList<QNetworkCookie> failed;
 };
@@ -355,39 +354,29 @@ void WebContext::UpdateUserScripts() {
       ->SerializeUserScriptsAndSendUpdates(scripts);
 }
 
-void WebContext::CookieSetCallback(
-    const scoped_refptr<SetCookiesContext>& ctxt,
-    const QNetworkCookie& cookie,
-    bool success) {
+void WebContext::SetCookieCallback(scoped_refptr<SetCookiesContext> ctxt,
+                                   const QNetworkCookie& cookie,
+                                   bool success) {
   DCHECK_GT(ctxt->remaining, 0);
 
   if (!success) {
     ctxt->failed.push_back(cookie);
   }
 
-  if (--ctxt->remaining > 0 || handling_cookie_request_) {
+  if (--ctxt->remaining > 0) {
     return;
   }
 
-  DeliverCookiesSet(ctxt);
+  DeliverSetCookiesResponse(ctxt);
 }
 
-void WebContext::DeliverCookiesSet(
-    const scoped_refptr<SetCookiesContext>& ctxt) {
-  DCHECK_EQ(ctxt->remaining, 0);
-  client_->CookiesSet(ctxt->id, ctxt->failed);
+void WebContext::DeliverSetCookiesResponse(
+    scoped_refptr<SetCookiesContext> ctxt) {
+  client_->CookiesSet(ctxt->request_id, ctxt->failed);
 }
 
-void WebContext::GotCookiesCallback(int request_id,
+void WebContext::GetCookiesCallback(int request_id,
                                     const net::CookieList& cookies) {
-  if (handling_cookie_request_) {
-    base::MessageLoop::current()->PostTask(
-        FROM_HERE,
-        base::Bind(&WebContext::GotCookiesCallback,
-                   weak_factory_.GetWeakPtr(), request_id, cookies));
-    return;
-  }
-
   QList<QNetworkCookie> qcookies;
   for (net::CookieList::const_iterator iter = cookies.begin();
        iter != cookies.end(); ++iter) {
@@ -410,16 +399,20 @@ void WebContext::GotCookiesCallback(int request_id,
   client_->CookiesRetrieved(request_id, qcookies);
 }
 
-void WebContext::DeletedCookiesCallback(int request_id,
-                                        int num_deleted) {
+void WebContext::DeleteCookiesCallback(int request_id, int num_deleted) {
   client_->CookiesDeleted(request_id, num_deleted);
 }
 
-WebContext::WebContext(WebContextProxyClient* client)
+WebContext::WebContext(WebContextProxyClient* client,
+                       QObject* handle)
     : client_(client),
       construct_props_(new ConstructProperties()),
-      handling_cookie_request_(false),
       weak_factory_(this) {
+  DCHECK(client);
+  DCHECK(handle);
+
+  setHandle(handle);
+
   delegate_ = new BrowserContextDelegate(weak_factory_.GetWeakPtr());
 
   static_assert(
@@ -456,10 +449,6 @@ WebContext::WebContext(WebContextProxyClient* client)
 }
 
 WebContext::~WebContext() {
-  if (g_default == this) {
-    g_default = nullptr;
-  }
-
   if (context_.get()) {
     context_->SetDelegate(nullptr);
     MediaCaptureDevicesContext::Get(context_.get())->set_client(nullptr);
@@ -477,26 +466,6 @@ WebContext* WebContext::FromBrowserContext(oxide::BrowserContext* context) {
   return delegate->context();
 }
 
-// static
-WebContext* WebContext::FromProxyHandle(WebContextProxyHandle* handle) {
-  return static_cast<WebContext*>(handle->proxy_.data());
-}
-
-// static
-WebContext* WebContext::GetDefault() {
-  return g_default;
-}
-
-// static
-void WebContext::DestroyDefault() {
-  if (!g_default || g_default_is_application_owned) {
-    return;
-  }
-
-  g_default->client_->DestroyDefault();
-  DCHECK(!g_default);
-}
-
 oxide::BrowserContext* WebContext::GetContext() {
   if (context_.get()) {
     return context_.get();
@@ -574,29 +543,9 @@ QNetworkAccessManager* WebContext::GetCustomNetworkAccessManager() {
 
 void WebContext::init(
     const QWeakPointer<WebContextProxyClient::IOClient>& io_client) {
-  // If we are in single process mode because it was set in the environment,
-  // allow the first application-created context to become the default
-  if (oxide::BrowserProcessMain::GetInstance()->GetProcessModel() ==
-          oxide::PROCESS_MODEL_SINGLE_PROCESS &&
-      BrowserStartup::GetInstance()->DidSelectProcessModelFromEnv() &&
-      !g_initialized_default) {
-    g_initialized_default = true;
-    g_default_is_application_owned = true;
-    g_default = this;
-  }
-
   delegate_->Init(io_client);
 }
 
-void WebContext::makeDefault() {
-  DCHECK(!g_initialized_default);
-  DCHECK(!g_default || g_default == this);
-
-  g_initialized_default = true;
-  g_default_is_application_owned = false;
-  g_default = this;
-}
-
 QString WebContext::product() const {
   if (IsInitialized()) {
     return QString::fromStdString(
@@ -695,7 +644,7 @@ void WebContext::setAcceptLangs(const QString& langs) {
   }
 }
 
-QList<UserScriptProxyHandle*>& WebContext::userScripts() {
+QList<QObject*>& WebContext::userScripts() {
   return user_scripts_;
 }
 
@@ -808,11 +757,9 @@ int WebContext::setCookies(const QUrl& url,
                            const QList<QNetworkCookie>& cookies) {
   int request_id = GetNextCookieRequestId();
 
-  base::AutoReset<bool> f(&handling_cookie_request_, true);
-
-  scoped_refptr<net::CookieStore> cookie_store = context_->GetCookieStore();
+  net::CookieStore* cookie_store = context_->GetCookieStore();
   scoped_refptr<SetCookiesContext> ctxt = new SetCookiesContext(request_id);
-
+  
   for (int i = 0; i < cookies.size(); ++i) {
     const QNetworkCookie& cookie = cookies.at(i);
 
@@ -828,27 +775,28 @@ int WebContext::setCookies(const QUrl& url,
 
     ctxt->remaining++;
 
-    cookie_store->GetCookieMonster()->SetCookieWithDetailsAsync(
+    cookie_store->SetCookieWithDetailsAsync(
         GURL(url.toString().toStdString()),
         std::string(cookie.name().constData()),
         std::string(cookie.value().constData()),
         std::string(cookie.domain().toUtf8().constData()),
         std::string(cookie.path().toUtf8().constData()),
+        base::Time(),
         expiry,
+        base::Time(),
         cookie.isSecure(),
         cookie.isHttpOnly(),
-        false, // first_party
-        false, // enforce_prefixes
+        net::CookieSameSite::DEFAULT_MODE,
         false, // enforce_strict_secure
         net::COOKIE_PRIORITY_DEFAULT,
-        base::Bind(&WebContext::CookieSetCallback,
+        base::Bind(&WebContext::SetCookieCallback,
                    weak_factory_.GetWeakPtr(), ctxt, cookie));
   }
 
   if (ctxt->remaining == 0) {
     base::MessageLoop::current()->PostTask(
         FROM_HERE,
-        base::Bind(&WebContext::DeliverCookiesSet,
+        base::Bind(&WebContext::DeliverSetCookiesResponse,
                    weak_factory_.GetWeakPtr(), ctxt));
   }
 
@@ -858,12 +806,9 @@ int WebContext::setCookies(const QUrl& url,
 int WebContext::getCookies(const QUrl& url) {
   int request_id = GetNextCookieRequestId();
 
-  base::AutoReset<bool> f(&handling_cookie_request_, true);
-
-  scoped_refptr<net::CookieStore> store = context_->GetCookieStore();
-  store->GetCookieMonster()->GetAllCookiesForURLAsync(
+  context_->GetCookieStore()->GetAllCookiesForURLAsync(
       GURL(url.toString().toStdString()),
-      base::Bind(&WebContext::GotCookiesCallback,
+      base::Bind(&WebContext::GetCookiesCallback,
                  weak_factory_.GetWeakPtr(), request_id));
 
   return request_id;
@@ -872,10 +817,8 @@ int WebContext::getCookies(const QUrl& url) {
 int WebContext::getAllCookies() {
   int request_id = GetNextCookieRequestId();
 
-  base::AutoReset<bool> f(&handling_cookie_request_, true);
-
-  context_->GetCookieStore()->GetCookieMonster()->GetAllCookiesAsync(
-      base::Bind(&WebContext::GotCookiesCallback,
+  context_->GetCookieStore()->GetAllCookiesAsync(
+      base::Bind(&WebContext::GetCookiesCallback,
                  weak_factory_.GetWeakPtr(), request_id));
 
   return request_id;
@@ -884,10 +827,8 @@ int WebContext::getAllCookies() {
 int WebContext::deleteAllCookies() {
   int request_id = GetNextCookieRequestId();
 
-  base::AutoReset<bool> f(&handling_cookie_request_, true);
-
-  context_->GetCookieStore()->GetCookieMonster()->DeleteAllAsync(
-      base::Bind(&WebContext::DeletedCookiesCallback,
+  context_->GetCookieStore()->DeleteAllAsync(
+      base::Bind(&WebContext::DeleteCookiesCallback,
                  weak_factory_.GetWeakPtr(), request_id));
 
   return request_id;
