@@ -17,8 +17,10 @@
 
 #include "oxide_compositor_gpu_shims.h"
 
+#include <memory>
+
+#include "base/lazy_instance.h"
 #include "base/single_thread_task_runner.h"
-#include "content/common/gpu/gpu_channel_manager.h"
 #include "content/gpu/gpu_child_thread.h"
 #include "gpu/command_buffer/service/context_group.h"
 #include "gpu/command_buffer/service/context_state.h"
@@ -26,14 +28,81 @@
 #include "gpu/command_buffer/service/mailbox_manager.h"
 #include "gpu/command_buffer/service/sync_point_manager.h"
 #include "gpu/command_buffer/service/texture_manager.h"
+#include "gpu/ipc/service/gpu_channel.h"
+#include "gpu/ipc/service/gpu_channel_manager.h"
+#include "gpu/ipc/service/gpu_command_buffer_stub.h"
 #include "ui/gl/gl_bindings.h"
 #include "ui/gl/gl_context.h"
 #include "ui/gl/gl_surface_egl.h"
 
-#include "shared/port/content/common/gpu_service_shim_oxide.h"
+#include "shared/port/gpu/gpu_service_shim_oxide.h"
 
 namespace oxide {
 
+class SyncPointWaiter {
+ public:
+  SyncPointWaiter();
+  ~SyncPointWaiter();
+
+  bool Wait(gpu::SyncPointClientState* release_state,
+            uint64_t release_count,
+            const base::Closure& wait_complete_callback);
+
+ private:
+  std::unique_ptr<gpu::SyncPointClient> sync_point_client_;
+};
+
+namespace {
+
+base::LazyInstance<SyncPointWaiter> g_sync_point_waiter =
+    LAZY_INSTANCE_INITIALIZER;
+
+gpu::gles2::GLES2Decoder* GetGLES2Decoder(
+    gpu::CommandBufferId command_buffer_id) {
+  int32_t client_id =
+      static_cast<int32_t>(command_buffer_id.GetUnsafeValue() >> 32);
+  int32_t route_id =
+      static_cast<int32_t>(command_buffer_id.GetUnsafeValue() &
+                           0x00000000FFFFFFFF);
+
+  gpu::GpuChannelManager* gpu_channel_manager =
+      content::GpuChildThread::GetChannelManager();
+  DCHECK(gpu_channel_manager);
+
+  gpu::GpuChannel* channel = gpu_channel_manager->LookupChannel(client_id);
+  if (!channel) {
+    return nullptr;
+  }
+
+  gpu::GpuCommandBufferStub* command_buffer =
+      channel->LookupCommandBuffer(route_id);
+  if (!command_buffer) {
+    return nullptr;
+  }
+
+  CHECK_EQ(command_buffer->command_buffer_id(), command_buffer_id);
+
+  return command_buffer->decoder();
+}
+
+}
+
+SyncPointWaiter::SyncPointWaiter()
+    : sync_point_client_(
+        content::GpuChildThread::GetChannelManager()
+          ->sync_point_manager()
+          ->CreateSyncPointClientWaiter()) {}
+
+SyncPointWaiter::~SyncPointWaiter() {}
+
+bool SyncPointWaiter::Wait(gpu::SyncPointClientState* release_state,
+                           uint64_t release_count,
+                           const base::Closure& wait_complete_callback) {
+  return sync_point_client_->WaitOutOfOrder(release_state,
+                                            release_count,
+                                            wait_complete_callback);
+}
+
 class ScopedTextureBinder {
  public:
   ScopedTextureBinder(GLenum target,
@@ -94,18 +163,34 @@ scoped_refptr<base::SingleThreadTaskRunner> GpuUtils::GetTaskRunner() {
 }
 
 // static
-bool GpuUtils::IsSyncPointRetired(uint32_t sync_point) {
-  return content::GpuChildThread::GetChannelManager()
-      ->sync_point_manager()
-      ->IsSyncPointRetired(sync_point);
+bool GpuUtils::IsSyncPointRetired(gpu::CommandBufferId command_buffer_id,
+                                  uint64_t sync_point) {
+  scoped_refptr<gpu::SyncPointClientState> client_state =
+      content::GpuChildThread::GetChannelManager()
+        ->sync_point_manager()
+        ->GetSyncPointClientState(gpu::CommandBufferNamespace::GPU_IO,
+                                  command_buffer_id);
+  if (!client_state) {
+    return true;
+  }
+
+  return client_state->IsFenceSyncReleased(sync_point);
 }
 
 // static
-void GpuUtils::AddSyncPointCallback(uint32_t sync_point,
-                                    const base::Closure& callback) {
-  content::GpuChildThread::GetChannelManager()
-      ->sync_point_manager()
-      ->AddSyncPointCallback(sync_point, callback);
+bool GpuUtils::WaitForSyncPoint(gpu::CommandBufferId command_buffer_id,
+                                uint64_t sync_point,
+                                const base::Closure& callback) {
+  scoped_refptr<gpu::SyncPointClientState> client_state =
+      content::GpuChildThread::GetChannelManager()
+        ->sync_point_manager()
+        ->GetSyncPointClientState(gpu::CommandBufferNamespace::GPU_IO,
+                                  command_buffer_id);
+  DCHECK(client_state);
+
+  return g_sync_point_waiter.Get().Wait(client_state.get(),
+                                        sync_point,
+                                        callback);
 }
 
 // static
@@ -121,11 +206,9 @@ EGLDisplay GpuUtils::GetHardwareEGLDisplay() {
 }
 
 // static
-GLuint GpuUtils::GetTextureFromMailbox(const CommandBufferID& command_buffer,
+GLuint GpuUtils::GetTextureFromMailbox(gpu::CommandBufferId command_buffer,
                                        const gpu::Mailbox& mailbox) {
-  gpu::gles2::GLES2Decoder* decoder =
-      content::oxide_gpu_shim::GetGLES2Decoder(
-        command_buffer.client_id, command_buffer.route_id);
+  gpu::gles2::GLES2Decoder* decoder = GetGLES2Decoder(command_buffer);
   if (!decoder) {
     return 0;
   }
@@ -141,11 +224,9 @@ GLuint GpuUtils::GetTextureFromMailbox(const CommandBufferID& command_buffer,
 
 // static
 EGLImageKHR GpuUtils::CreateEGLImageFromMailbox(
-    const CommandBufferID& command_buffer,
+    gpu::CommandBufferId command_buffer,
     const gpu::Mailbox& mailbox) {
-  gpu::gles2::GLES2Decoder* decoder =
-      content::oxide_gpu_shim::GetGLES2Decoder(
-        command_buffer.client_id, command_buffer.route_id);
+  gpu::gles2::GLES2Decoder* decoder = GetGLES2Decoder(command_buffer);
   if (!decoder) {
     return EGL_NO_IMAGE_KHR;
   }
