@@ -28,6 +28,7 @@
 #include "base/files/file_util.h"
 #include "base/lazy_instance.h"
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/memory/weak_ptr.h"
 #include "base/supports_user_data.h"
 #include "base/synchronization/lock.h"
@@ -41,7 +42,6 @@
 #include "content/public/browser/storage_partition.h"
 #include "net/base/host_mapping_rules.h"
 #include "net/base/net_errors.h"
-#include "net/base/net_util.h"
 #include "net/cookies/cookie_monster.h"
 #include "net/ftp/ftp_network_layer.h"
 #include "net/http/http_cache.h"
@@ -60,6 +60,8 @@
 
 #include "shared/browser/permissions/oxide_permission_manager.h"
 #include "shared/browser/permissions/oxide_temporary_saved_permission_context.h"
+#include "shared/browser/ssl/oxide_ssl_config_service.h"
+#include "shared/browser/ssl/oxide_ssl_host_state_delegate.h"
 #include "shared/common/oxide_constants.h"
 #include "shared/common/oxide_content_client.h"
 
@@ -67,12 +69,11 @@
 #include "oxide_browser_context_destroyer.h"
 #include "oxide_browser_context_observer.h"
 #include "oxide_browser_process_main.h"
+#include "oxide_cookie_store_ui_proxy.h"
 #include "oxide_download_manager_delegate.h"
 #include "oxide_http_user_agent_settings.h"
 #include "oxide_io_thread.h"
 #include "oxide_network_delegate.h"
-#include "oxide_ssl_config_service.h"
-#include "oxide_ssl_host_state_delegate.h"
 #include "oxide_url_request_context.h"
 #include "oxide_url_request_delegated_job_factory.h"
 #include "oxide_user_agent_settings.h"
@@ -207,7 +208,7 @@ struct BrowserContextSharedIOData {
 
   std::vector<std::string> host_mapping_rules;
 
-  scoped_ptr<UserAgentSettingsIOData> user_agent_settings;
+  std::unique_ptr<UserAgentSettingsIOData> user_agent_settings;
 
   bool do_not_track;
 
@@ -282,6 +283,11 @@ class OTRBrowserContextIODataImpl : public BrowserContextIOData {
 };
 
 void BrowserContextIOData::Init() {
+  // FIXME(chrisccoulson): net::CookieStore is not thread-safe - we need to be
+  //  creating this on the IO thread. Whilst this is sort-of ok for now (we
+  //  guarantee that there are no concurrent accesses here), it will break
+  //  if net::CookieMonster is modified to assert that it's only accessed on
+  //  a single thread
   DCHECK(!cookie_store_.get());
 
   base::FilePath cookie_path;
@@ -402,14 +408,14 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
   // TODO: We want persistent storage here (for non-incognito), but 
   //       SQLiteChannelIDStore is part of chrome
   storage->set_channel_id_service(
-      make_scoped_ptr(new net::ChannelIDService(
+      base::WrapUnique(new net::ChannelIDService(
           new net::DefaultChannelIDStore(nullptr),
           base::WorkerPool::GetTaskRunner(true))));
 
   context->set_http_server_properties(http_server_properties_->GetWeakPtr());
 
   DCHECK(cookie_store_.get());
-  storage->set_cookie_store(cookie_store_.get());
+  context->set_cookie_store(cookie_store_.get());
 
   context->set_transport_security_state(transport_security_state_.get());
 
@@ -420,7 +426,7 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
       FROM_HERE,
       base::Bind(&CleanupOldCacheDir, GetCachePath().Append(kCacheDirname)));
 
-  scoped_ptr<net::HttpCache::BackendFactory> cache_backend;
+  std::unique_ptr<net::HttpCache::BackendFactory> cache_backend;
   if (IsOffTheRecord() || GetCachePath().empty()) {
     cache_backend = net::HttpCache::DefaultBackend::InMemory(0);
   } else {
@@ -443,25 +449,25 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
   session_params.ssl_config_service = context->ssl_config_service();
   session_params.http_auth_handler_factory =
       context->http_auth_handler_factory();
-  session_params.network_delegate = context->network_delegate();
   session_params.http_server_properties =
       context->http_server_properties();
   session_params.net_log = context->net_log();
   session_params.host_mapping_rules = host_mapping_rules_.get();
 
   http_network_session_ =
-      make_scoped_ptr(new net::HttpNetworkSession(session_params));
+      base::WrapUnique(new net::HttpNetworkSession(session_params));
 
   {
     // Calls QuickStreamFactory constructor which uses base::CPU
     base::ThreadRestrictions::ScopedAllowIO allow_io;
     storage->set_http_transaction_factory(
-        make_scoped_ptr(new net::HttpCache(http_network_session_.get(),
-                                           std::move(cache_backend),
-                                           true)));
+        base::WrapUnique(
+            new net::HttpCache(http_network_session_.get(),
+                               std::move(cache_backend),
+                               true)));
   }
 
-  scoped_ptr<net::URLRequestJobFactoryImpl> job_factory(
+  std::unique_ptr<net::URLRequestJobFactoryImpl> job_factory(
       new net::URLRequestJobFactoryImpl());
 
   bool set_protocol = false;
@@ -470,29 +476,30 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
        ++it) {
     set_protocol =
         job_factory->SetProtocolHandler(it->first,
-                                        make_scoped_ptr(it->second.release()));
+                                        base::WrapUnique(it->second.release()));
     DCHECK(set_protocol);
   }
   protocol_handlers.clear();
 
   set_protocol = job_factory->SetProtocolHandler(
       oxide::kFileScheme,
-      make_scoped_ptr(new net::FileProtocolHandler(
-        content::BrowserThread::GetMessageLoopProxyForThread(
-          content::BrowserThread::FILE))));
+      base::WrapUnique(
+          new net::FileProtocolHandler(
+              content::BrowserThread::GetMessageLoopProxyForThread(
+                  content::BrowserThread::FILE))));
   DCHECK(set_protocol);
   set_protocol = job_factory->SetProtocolHandler(
       oxide::kDataScheme,
-      make_scoped_ptr(new net::DataProtocolHandler()));
+      base::WrapUnique(new net::DataProtocolHandler()));
   DCHECK(set_protocol);
 
   set_protocol = job_factory->SetProtocolHandler(
       oxide::kFtpScheme,
-      make_scoped_ptr(new net::FtpProtocolHandler(
+      base::WrapUnique(new net::FtpProtocolHandler(
         ftp_transaction_factory_.get())));
   DCHECK(set_protocol);
 
-  scoped_ptr<net::URLRequestJobFactory> top_job_factory(
+  std::unique_ptr<net::URLRequestJobFactory> top_job_factory(
       new URLRequestDelegatedJobFactory(std::move(job_factory),
                                         this));
 
@@ -500,8 +507,9 @@ URLRequestContext* BrowserContextIOData::CreateMainRequestContext(
           request_interceptors.rbegin();
        it != request_interceptors.rend();
        ++it) {
-    top_job_factory.reset(new net::URLRequestInterceptingJobFactory(
-        std::move(top_job_factory), make_scoped_ptr(*it)));
+    top_job_factory.reset(
+        new net::URLRequestInterceptingJobFactory(std::move(top_job_factory),
+                                                  base::WrapUnique(*it)));
   }
   request_interceptors.weak_clear();
 
@@ -535,6 +543,11 @@ UserAgentSettingsIOData* BrowserContextIOData::GetUserAgentSettings() const {
   return GetSharedData().user_agent_settings.get();
 }
 
+net::CookieStore* BrowserContextIOData::GetCookieStore() const {
+  DCHECK_CURRENTLY_ON(content::BrowserThread::IO);
+  return cookie_store_.get();
+}
+
 class BrowserContextImpl;
 
 class OTRBrowserContextImpl : public BrowserContext {
@@ -630,7 +643,8 @@ void BrowserContextTraits::Destruct(const BrowserContext* x) {
   BrowserContextDestroyer::DestroyContext(const_cast<BrowserContext*>(x));
 }
 
-scoped_ptr<content::ZoomLevelDelegate> BrowserContext::CreateZoomLevelDelegate(
+std::unique_ptr<content::ZoomLevelDelegate>
+BrowserContext::CreateZoomLevelDelegate(
     const base::FilePath& partition_path) {
   return nullptr;
 }
@@ -640,16 +654,6 @@ net::URLRequestContextGetter* BrowserContext::GetRequestContext() {
   return GetStoragePartition(this, nullptr)->GetURLRequestContext();
 }
 
-net::URLRequestContextGetter*
-BrowserContext::GetRequestContextForRenderProcess(int renderer_child_id) {
-  DCHECK(CalledOnValidThread());
-
-  content::RenderProcessHost* host =
-      content::RenderProcessHost::FromID(renderer_child_id);
-
-  return host->GetStoragePartition()->GetURLRequestContext();
-}
-
 net::URLRequestContextGetter* BrowserContext::GetMediaRequestContext() {
   DCHECK(CalledOnValidThread());
   return GetRequestContext();
@@ -714,6 +718,32 @@ BrowserContext::GetBackgroundSyncController() {
   return nullptr;
 }
 
+net::URLRequestContextGetter* BrowserContext::CreateRequestContext(
+    content::ProtocolHandlerMap* protocol_handlers,
+    content::URLRequestInterceptorScopedVector request_interceptors) {
+  DCHECK(CalledOnValidThread());
+  DCHECK(!main_request_context_getter_.get());
+
+  main_request_context_getter_ =
+      new MainURLRequestContextGetter(io_data(),
+                                      protocol_handlers,
+                                      std::move(request_interceptors));
+
+  return main_request_context_getter_.get();
+}
+
+net::URLRequestContextGetter*
+BrowserContext::CreateRequestContextForStoragePartition(
+    const base::FilePath& partition_path,
+    bool in_memory,
+    content::ProtocolHandlerMap* protocol_handlers,
+    content::URLRequestInterceptorScopedVector request_interceptors) {
+  // We don't return any storage partition names from
+  // GetStoragePartitionConfigForSite(), so it's a bug to hit this
+  NOTREACHED() << "Invalid request for request context for storage partition";
+  return nullptr;
+}
+
 void BrowserContext::AddObserver(BrowserContextObserver* observer) {
   DCHECK(CalledOnValidThread());
   observers_.AddObserver(observer);
@@ -734,7 +764,9 @@ BrowserContext::BrowserContext(BrowserContextIOData* io_data) :
 
   // Make sure that the cookie store is properly created
   io_data->Init();
+  cookie_store_.reset(new CookieStoreUIProxy(io_data->cookie_store_.get()));
 
+  content::BrowserContext::Initialize(this, io_data->GetPath());
   content::BrowserContext::EnsureResourceContextInitialized(this);
 }
 
@@ -778,20 +810,6 @@ BrowserContextID BrowserContext::GetID() const {
   return reinterpret_cast<BrowserContextID>(this);
 }
 
-net::URLRequestContextGetter* BrowserContext::CreateRequestContext(
-    content::ProtocolHandlerMap* protocol_handlers,
-    content::URLRequestInterceptorScopedVector request_interceptors) {
-  DCHECK(CalledOnValidThread());
-  DCHECK(!main_request_context_getter_.get());
-
-  main_request_context_getter_ =
-      new MainURLRequestContextGetter(io_data(),
-                                      protocol_handlers,
-                                      std::move(request_interceptors));
-
-  return main_request_context_getter_.get();
-}
-
 BrowserContextDelegate* BrowserContext::GetDelegate() const {
   DCHECK(CalledOnValidThread());
   return io_data()->GetDelegate().get();
@@ -881,18 +899,21 @@ content::ResourceContext* BrowserContext::GetResourceContext() {
   return io_data()->GetResourceContext();
 }
 
-scoped_refptr<net::CookieStore> BrowserContext::GetCookieStore() {
-  DCHECK(CalledOnValidThread());
-  return io_data()->cookie_store_;
+net::CookieStore* BrowserContext::GetCookieStore() const {
+  return cookie_store_.get();
 }
 
-
 TemporarySavedPermissionContext*
 BrowserContext::GetTemporarySavedPermissionContext() const {
   DCHECK(CalledOnValidThread());
   return io_data()->GetTemporarySavedPermissionContext();
 }
 
+BrowserContextIOData* BrowserContext::GetIOData() const {
+  DCHECK(CalledOnValidThread());
+  return io_data_;
+}
+
 bool BrowserContext::GetDoNotTrack() const {
   DCHECK(CalledOnValidThread());
   return io_data()->GetSharedData().do_not_track;
