@@ -21,6 +21,7 @@
 
 #include "base/files/file_path.h"
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
 #include "base/scoped_native_library.h"
 #include "content/browser/gpu/gpu_data_manager_impl.h"
@@ -32,7 +33,6 @@
 #include "third_party/WebKit/public/platform/WebScreenInfo.h"
 #include "ui/gfx/display.h"
 #include "ui/gfx/screen.h"
-#include "ui/gfx/screen_type_delegate.h"
 #include "ui/gl/gl_context.h"
 #include "ui/gl/gl_surface.h"
 
@@ -44,8 +44,8 @@
 #include "shared/port/content/browser/power_save_blocker_oxide.h"
 #include "shared/port/content/browser/render_widget_host_view_oxide.h"
 #include "shared/port/content/browser/web_contents_view_oxide.h"
-#include "shared/port/content/common/gpu_service_shim_oxide.h"
 #include "shared/port/gfx/gfx_utils_oxide.h"
+#include "shared/port/gpu/gpu_service_shim_oxide.h"
 #include "shared/port/gpu_config/gpu_info_collector_oxide_linux.h"
 #include "shared/port/media/video_capture_device_factory_override.h"
 #include "shared/port/ui_base/clipboard_oxide.h"
@@ -60,19 +60,16 @@
 #include "oxide_message_pump.h"
 #include "oxide_power_save_blocker.h"
 #include "oxide_render_process_initializer.h"
+#include "oxide_screen_client.h"
 #include "oxide_web_contents_view.h"
 
 namespace oxide {
 
 namespace {
 
-blink::WebScreenInfo DefaultScreenInfoGetter() {
-  return BrowserPlatformIntegration::GetInstance()->GetDefaultScreenInfo();
-}
-
-scoped_ptr<media::VideoCaptureDeviceFactory> CreateVideoCaptureDeviceFactory(
-    scoped_ptr<media::VideoCaptureDeviceFactory> delegate) {
-  return make_scoped_ptr(
+std::unique_ptr<media::VideoCaptureDeviceFactory> CreateVideoCaptureDeviceFactory(
+    std::unique_ptr<media::VideoCaptureDeviceFactory> delegate) {
+  return base::WrapUnique(
       new VideoCaptureDeviceFactoryLinux(std::move(delegate)));
 }
 
@@ -80,7 +77,7 @@ ui::Clipboard* CreateClipboard() {
   return BrowserPlatformIntegration::GetInstance()->CreateClipboard();
 }
 
-scoped_ptr<base::MessagePump> CreateUIMessagePump() {
+std::unique_ptr<base::MessagePump> CreateUIMessagePump() {
   return BrowserPlatformIntegration::GetInstance()->CreateUIMessagePump();
 }
 
@@ -161,27 +158,27 @@ class Screen : public gfx::Screen {
   Screen() {}
 
   gfx::Point GetCursorScreenPoint() final {
-    NOTIMPLEMENTED();
+    NOTREACHED();
     return gfx::Point();
   }
 
   gfx::NativeWindow GetWindowUnderCursor() final {
-    NOTIMPLEMENTED();
+    NOTREACHED();
     return nullptr;
   }
 
   gfx::NativeWindow GetWindowAtScreenPoint(const gfx::Point& point) final {
-    NOTIMPLEMENTED();
+    NOTREACHED();
     return nullptr;
   }
 
   int GetNumDisplays() const final {
-    NOTIMPLEMENTED();
+    NOTREACHED();
     return 1;
   }
 
   std::vector<gfx::Display> GetAllDisplays() const final {
-    NOTIMPLEMENTED();
+    NOTREACHED();
     return std::vector<gfx::Display>();
   }
 
@@ -190,43 +187,36 @@ class Screen : public gfx::Screen {
     //  is the NativeView for the corresponding RenderWidgetHostView. It would
     //  be nice to find a way to cleverly map this to the associated RWHV and
     //  get the correct display
-    return GetPrimaryDisplay();
+    return gfx::Display();
   }
 
   gfx::Display GetDisplayNearestPoint(const gfx::Point& point) const final {
-    NOTIMPLEMENTED();
+    NOTREACHED();
     return gfx::Display();
   }
 
   gfx::Display GetDisplayMatching(const gfx::Rect& match_rect) const final {
-    NOTIMPLEMENTED();
+    NOTREACHED();
     return gfx::Display();
   }
 
   gfx::Display GetPrimaryDisplay() const final {
-    blink::WebScreenInfo info(
-        BrowserPlatformIntegration::GetInstance()->GetDefaultScreenInfo());
-
-    gfx::Display display;
-    display.set_bounds(info.rect);
-    display.set_work_area(info.availableRect);
-    display.set_device_scale_factor(info.deviceScaleFactor);
-
-    return display;
+    return BrowserPlatformIntegration::GetInstance()
+        ->GetScreenClient()
+        ->GetPrimaryDisplay();
   }
 
   void AddObserver(gfx::DisplayObserver* observer) final {
-    NOTIMPLEMENTED();
+    NOTREACHED();
   }
   void RemoveObserver(gfx::DisplayObserver* observer) final {
-    NOTIMPLEMENTED();
+    NOTREACHED();
   }
 };
 
 } // namespace
 
 void BrowserMainParts::PreEarlyInitialization() {
-  content::SetDefaultScreenInfoGetterOxide(DefaultScreenInfoGetter);
   content::SetWebContentsViewOxideFactory(WebContentsView::Create);
   content::SetPowerSaveBlockerOxideDelegateFactory(CreatePowerSaveBlocker);
   media::SetVideoCaptureDeviceFactoryOverrideFactory(
@@ -259,8 +249,7 @@ int BrowserMainParts::PreCreateThreads() {
   }
 
   primary_screen_.reset(new Screen());
-  gfx::Screen::SetScreenInstance(gfx::SCREEN_TYPE_NATIVE,
-                                 primary_screen_.get());
+  gfx::Screen::SetScreenInstance(primary_screen_.get());
 
   io_thread_.reset(new IOThread());
 
@@ -282,7 +271,7 @@ void BrowserMainParts::PreMainMessageLoopRun() {
       // that you can assume it's safe to use the handle returned by it for
       // anything
       gl_share_context_ = GLContextDependent::CloneFrom(share_context.get());
-      content::oxide_gpu_shim::SetGLShareGroup(gl_share_context_->share_group());
+      gpu::oxide_shim::SetGLShareGroup(gl_share_context_->share_group());
     }
   }
 
@@ -328,10 +317,10 @@ void BrowserMainParts::PostDestroyThreads() {
     BrowserContext::AssertNoContextsExist();
   }
 
-  gfx::Screen::SetScreenInstance(gfx::SCREEN_TYPE_NATIVE, nullptr);
+  gfx::Screen::SetScreenInstance(nullptr);
   io_thread_.reset();
 
-  content::oxide_gpu_shim::SetGLShareGroup(nullptr);
+  gpu::oxide_shim::SetGLShareGroup(nullptr);
   gl_share_context_ = nullptr;
 
   gpu::SetGpuInfoCollectorOxideLinux(nullptr);
