@@ -21,6 +21,7 @@
 #include "content/public/browser/location_provider.h"
 
 #include "oxide_browser_platform_integration_observer.h"
+#include "oxide_drag_source.h"
 
 namespace oxide {
 
@@ -71,7 +72,7 @@ void BrowserPlatformIntegration::BrowserThreadInit(
 void BrowserPlatformIntegration::BrowserThreadCleanUp(
     content::BrowserThread::ID id) {}
 
-scoped_ptr<content::LocationProvider>
+std::unique_ptr<content::LocationProvider>
 BrowserPlatformIntegration::CreateLocationProvider() {
   return nullptr;
 }
@@ -85,6 +86,11 @@ std::string BrowserPlatformIntegration::GetApplicationName() {
   return kDefaultApplicationName;
 }
 
+std::unique_ptr<DragSource> BrowserPlatformIntegration::CreateDragSource(
+    DragSourceClient* client) {
+  return nullptr;
+}
+
 void BrowserPlatformIntegration::AddObserver(
     BrowserPlatformIntegrationObserver* observer) {
   observers_.AddObserver(observer);
@@ -107,4 +113,8 @@ void BrowserPlatformIntegration::NotifyClipboardDataChanged() {
                     ClipboardDataChanged());
 }
 
+void BrowserPlatformIntegration::CreateVibrationManager(
+      mojo::InterfaceRequest<device::VibrationManager> request) {
+}
+
 } // namespace oxide
