@@ -18,13 +18,15 @@
 #ifndef _OXIDE_SHARED_BROWSER_PLATFORM_INTEGRATION_H_
 #define _OXIDE_SHARED_BROWSER_PLATFORM_INTEGRATION_H_
 
+#include <memory>
 #include <string>
 
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/observer_list.h"
 #include "content/public/browser/browser_thread.h"
-#include "third_party/WebKit/public/platform/WebScreenInfo.h"
+#include "device/vibration/vibration_manager_impl.h"
+
+#include "shared/common/oxide_shared_export.h"
 
 class GURL;
 
@@ -39,12 +41,15 @@ class Clipboard;
 namespace oxide {
 
 class BrowserPlatformIntegrationObserver;
+class DragSource;
+class DragSourceClient;
 class GLContextDependent;
 class MessagePump;
+class ScreenClient;
 
 // An abstract interface allowing toolkit-independent code to integrate with
 // toolkit-specific features
-class BrowserPlatformIntegration {
+class OXIDE_SHARED_EXPORT BrowserPlatformIntegration {
  public:
   virtual ~BrowserPlatformIntegration();
 
@@ -76,8 +81,9 @@ class BrowserPlatformIntegration {
   // Called on the UI thread
   virtual intptr_t GetNativeDisplay() = 0;
 
-  // Return information about the default screen. Can be called on any thread
-  virtual blink::WebScreenInfo GetDefaultScreenInfo() = 0;
+  // Get the ScreenClient implementation. Can be called on any thread
+  // FIXME(chrisccoulson): Make this UI thread only
+  virtual ScreenClient* GetScreenClient() = 0;
 
   // Return the shared GL context provided by the application, if one exists.
   // This will be used for sharing resources between the webview and UI
@@ -86,7 +92,7 @@ class BrowserPlatformIntegration {
 
   // Create a MessagePump that allows Chromium events to be pumped from
   // the applications UI event loop
-  virtual scoped_ptr<MessagePump> CreateUIMessagePump() = 0;
+  virtual std::unique_ptr<MessagePump> CreateUIMessagePump() = 0;
 
   // Create a ui::Clipboard implementation. Can return nullptr
   virtual ui::Clipboard* CreateClipboard();
@@ -97,14 +103,27 @@ class BrowserPlatformIntegration {
 
   // Create a LocationProvider for determining location information from
   // the toolkit. Can return nullptr. Called on the geolocation thread
-  virtual scoped_ptr<content::LocationProvider> CreateLocationProvider();
+  virtual std::unique_ptr<content::LocationProvider> CreateLocationProvider();
 
   // Get the current application state. Called on the UI thread
   virtual ApplicationState GetApplicationState();
 
+  // Time limit, in milliseconds, to distinguish between aggregate or just
+  // successive mouse click events. 
+  virtual int GetClickInterval() = 0;
+
   // Get the application name. Can be called on any thread
   virtual std::string GetApplicationName();
 
+  // Create a new DragSource implementation. Ownership of |client| is not
+  // transferred, and |client| will outlive the returned DragSource.
+  // Called on the UI thread
+  virtual std::unique_ptr<DragSource> CreateDragSource(DragSourceClient* client);
+
+  // Get the proper vibration manager factory
+  virtual void CreateVibrationManager(
+      mojo::InterfaceRequest<device::VibrationManager> request);
+
  protected:
   BrowserPlatformIntegration();
 
