@@ -25,6 +25,7 @@
 
 #include "base/at_exit.h"
 #include "base/base_paths.h"
+#include "base/base_switches.h"
 #include "base/command_line.h"
 #include "base/files/file_path.h"
 #include "base/files/file_util.h"
@@ -125,10 +126,10 @@ class BrowserProcessMainImpl : public BrowserProcessMain {
   ProcessModel process_model_;
 
   // XXX: Don't change the order of these
-  scoped_ptr<PlatformDelegate> platform_delegate_;
-  scoped_ptr<ContentMainDelegate> main_delegate_;
-  scoped_ptr<base::AtExitManager> exit_manager_;
-  scoped_ptr<content::BrowserMainRunner> browser_main_runner_;
+  std::unique_ptr<PlatformDelegate> platform_delegate_;
+  std::unique_ptr<ContentMainDelegate> main_delegate_;
+  std::unique_ptr<base::AtExitManager> exit_manager_;
+  std::unique_ptr<content::BrowserMainRunner> browser_main_runner_;
 };
 
 namespace {
@@ -244,6 +245,9 @@ void InitializeCommandLine(const base::FilePath& subprocess_path,
   command_line->AppendSwitch(switches::kUIPrioritizeInGpuProcess);
   command_line->AppendSwitch(switches::kEnableSmoothScrolling);
 
+  command_line->AppendSwitchASCII(switches::kProfilerTiming,
+                                  switches::kProfilerTimingDisabledValue);
+
   if (gl_impl == gfx::kGLImplementationNone ||
       IsEnvironmentOptionEnabled("DISABLE_GPU")) {
     command_line->AppendSwitch(switches::kDisableGpu);
@@ -341,13 +345,12 @@ void AddFormFactorSpecificCommandLineArguments() {
 
 bool IsUnsupportedProcessModel(ProcessModel process_model) {
   switch (process_model) {
+    case PROCESS_MODEL_MULTI_PROCESS:
     case PROCESS_MODEL_SINGLE_PROCESS:
-    case PROCESS_MODEL_PROCESS_PER_VIEW:
-    case PROCESS_MODEL_PROCESS_PER_SITE:
-    case PROCESS_MODEL_SITE_PER_PROCESS:
-      return true;
-    default:
+    case PROCESS_MODEL_PROCESS_PER_SITE_INSTANCE:
       return false;
+    default:
+      return true;
   }
 }
 
@@ -368,7 +371,7 @@ const char* GetFormFactorHintCommandLine(FormFactor form_factor) {
 }
 
 BrowserProcessMain::StartParams::StartParams(
-    scoped_ptr<PlatformDelegate> delegate)
+    std::unique_ptr<PlatformDelegate> delegate)
     : delegate(std::move(delegate)),
       gl_implementation(gfx::kGLImplementationNone),
       process_model(PROCESS_MODEL_MULTI_PROCESS) {}
@@ -382,7 +385,7 @@ BrowserProcessMain::StartParams::StartParams(StartParams&& other)
 #endif
       gl_implementation(std::move(other.gl_implementation)),
       process_model(std::move(other.process_model)),
-      primary_screen_size_dip(std::move(other.primary_screen_size_dip)) {}
+      primary_screen_size(std::move(other.primary_screen_size)) {}
 
 BrowserProcessMainImpl::BrowserProcessMainImpl()
     : state_(STATE_NOT_STARTED),
@@ -423,7 +426,7 @@ void BrowserProcessMainImpl::Start(StartParams params) {
                         params.gl_implementation);
 
   FormFactor form_factor =
-      DetectFormFactorHint(params.primary_screen_size_dip);
+      DetectFormFactorHint(params.primary_screen_size);
   base::CommandLine::ForCurrentProcess()
       ->AppendSwitchASCII(switches::kFormFactor,
                           GetFormFactorHintCommandLine(form_factor));
