@@ -22,21 +22,24 @@
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "base/memory/scoped_vector.h"
-#include "cc/layers/delegated_frame_provider.h"
-#include "cc/layers/delegated_renderer_layer.h"
-#include "cc/layers/layer_settings.h"
+#include "cc/layers/surface_layer.h"
 #include "cc/output/compositor_frame.h"
 #include "cc/output/compositor_frame_ack.h"
 #include "cc/output/delegated_frame_data.h"
 #include "cc/output/viewport_selection_bound.h"
 #include "cc/quads/render_pass.h"
+#include "cc/surfaces/surface.h"
+#include "cc/surfaces/surface_id_allocator.h"
+#include "cc/surfaces/surface_manager.h"
 #include "content/browser/renderer_host/render_widget_host_delegate.h"
 #include "content/browser/renderer_host/render_widget_host_impl.h"
 #include "content/public/browser/browser_thread.h"
 #include "content/public/browser/render_process_host.h"
 #include "content/public/browser/render_view_host.h"
 #include "content/public/common/content_switches.h"
+#include "content/public/common/context_menu_params.h"
 #include "third_party/WebKit/public/platform/WebCursorInfo.h"
 #include "third_party/WebKit/public/platform/WebGestureDevice.h"
 #include "third_party/WebKit/public/web/WebInputEvent.h"
@@ -48,12 +51,12 @@
 #include "ui/touch_selection/touch_selection_controller.h"
 
 #include "shared/browser/compositor/oxide_compositor.h"
+#include "shared/browser/compositor/oxide_compositor_utils.h"
 #include "shared/browser/input/oxide_input_method_context.h"
 
 #include "oxide_browser_platform_integration.h"
 #include "oxide_browser_process_main.h"
 #include "oxide_event_utils.h"
-#include "oxide_renderer_frame_evictor.h"
 #include "oxide_render_widget_host_view_container.h"
 
 namespace oxide {
@@ -112,6 +115,36 @@ ui::SelectionBound ConvertSelectionBound(
   return ui_bound;
 }
 
+void SatisfyCallback(cc::SurfaceManager* manager,
+                     cc::SurfaceSequence sequence) {
+  std::vector<uint32_t> sequences;
+  sequences.push_back(sequence.sequence);
+  manager->DidSatisfySequences(sequence.id_namespace, &sequences);
+}
+
+void RequireCallback(cc::SurfaceManager* manager,
+                     cc::SurfaceId id,
+                     cc::SurfaceSequence sequence) {
+  cc::Surface* surface = manager->GetSurfaceForId(id);
+  if (!surface) {
+    LOG(ERROR) << "Attempting to require callback on nonexistent surface";
+    return;
+  }
+  surface->AddDestructionDependency(sequence);
+}
+
+bool HasLocationBarOffsetChanged(const cc::CompositorFrameMetadata& old,
+                                 const cc::CompositorFrameMetadata& current) {
+  if (old.location_bar_offset.y() != current.location_bar_offset.y()) {
+    return true;
+  }
+  if (old.location_bar_content_translation.y() !=
+      current.location_bar_content_translation.y()) {
+    return true;
+  }
+  return false;
+}
+
 } // namespace
 
 void RenderWidgetHostView::OnTextInputStateChanged(
@@ -178,7 +211,7 @@ gfx::Size RenderWidgetHostView::GetPhysicalBackingSize() const {
     return gfx::Size();
   }
 
-  return container_->GetViewSizePix();
+  return container_->GetViewSizeInPixels();
 }
 
 bool RenderWidgetHostView::DoTopControlsShrinkBlinkSize() const {
@@ -194,7 +227,7 @@ float RenderWidgetHostView::GetTopControlsHeight() const {
     return 0.0f;
   }
 
-  return container_->GetLocationBarHeightDip();
+  return container_->GetLocationBarHeight();
 }
 
 void RenderWidgetHostView::FocusedNodeChanged(bool is_editable_node) {
@@ -203,15 +236,14 @@ void RenderWidgetHostView::FocusedNodeChanged(bool is_editable_node) {
 
 void RenderWidgetHostView::OnSwapCompositorFrame(
     uint32_t output_surface_id,
-    scoped_ptr<cc::CompositorFrame> frame) {
+    std::unique_ptr<cc::CompositorFrame> frame) {
   if (!frame->delegated_frame_data) {
     DLOG(ERROR) << "Non delegated renderer path is not supported";
     host_->GetProcess()->ShutdownForBadMessage();
     return;
   }
 
-  scoped_ptr<cc::DelegatedFrameData> frame_data =
-      std::move(frame->delegated_frame_data);
+  cc::DelegatedFrameData* frame_data = frame->delegated_frame_data.get();
 
   if (frame_data->render_pass_list.empty()) {
     DLOG(ERROR) << "Invalid delegated frame";
@@ -220,23 +252,18 @@ void RenderWidgetHostView::OnSwapCompositorFrame(
   }
 
   if (output_surface_id != last_output_surface_id_) {
-    resource_collection_->SetClient(nullptr);
-    if (resource_collection_->LoseAllResources()) {
+    DestroyDelegatedContent();
+    surface_factory_.reset();
+    if (!surface_returned_resources_.empty()) {
       SendReturnedDelegatedResources();
     }
-    resource_collection_ = new cc::DelegatedFrameResourceCollection();
-    resource_collection_->SetClient(this);
-
-    DestroyDelegatedContent();
-
-    // XXX(chrisccoulson): Should we clear ack_callbacks_ here as well?
-
     last_output_surface_id_ = output_surface_id;
   }
 
   base::Closure ack_callback =
       base::Bind(&RenderWidgetHostView::SendDelegatedFrameAck,
-                 AsWeakPtr(), output_surface_id);
+                 weak_ptr_factory_.GetWeakPtr(),
+                 output_surface_id);
   ack_callbacks_.push(ack_callback);
 
   float device_scale_factor = frame->metadata.device_scale_factor;
@@ -251,61 +278,75 @@ void RenderWidgetHostView::OnSwapCompositorFrame(
       gfx::ScaleRect(gfx::RectF(root_pass->damage_rect),
                      1.0f / device_scale_factor));
 
+  cc::CompositorFrameMetadata metadata = frame->metadata;
+
   if (frame_size.IsEmpty()) {
     DestroyDelegatedContent();
   } else {
-    if (!frame_provider_.get() ||
-        frame_size != frame_provider_->frame_size() ||
-        frame_size_dip != last_frame_size_dip_) {
-      DetachLayer();
+    cc::SurfaceManager* manager =
+        CompositorUtils::GetInstance()->GetSurfaceManager();
+    if (!surface_factory_) {
+      DCHECK(manager);
+      surface_factory_ =
+          base::WrapUnique(new cc::SurfaceFactory(manager, this));
+    }
 
-      frame_provider_ = new cc::DelegatedFrameProvider(resource_collection_,
-                                                       std::move(frame_data));
-      layer_ = cc::DelegatedRendererLayer::Create(cc::LayerSettings(),
-                                                  frame_provider_);
+    if (surface_id_.is_null() ||
+        frame_size_dip != last_frame_size_dip_) {
+      DestroyDelegatedContent();
+
+      surface_id_ = id_allocator_->GenerateId();
+      DCHECK(!surface_id_.is_null());
+      surface_factory_->Create(surface_id_);
+
+      layer_ =
+          cc::SurfaceLayer::Create(base::Bind(&SatisfyCallback,
+                                              base::Unretained(manager)),
+                                   base::Bind(&RequireCallback,
+                                              base::Unretained(manager)));
+      DCHECK(layer_);
+      layer_->SetSurfaceId(surface_id_, device_scale_factor, frame_size);
+      layer_->SetBounds(frame_size_dip);
       layer_->SetIsDrawable(true);
       layer_->SetContentsOpaque(true);
-      layer_->SetBounds(frame_size_dip);
       layer_->SetHideLayerAndSubtree(!is_showing_);
 
       AttachLayer();
-    } else {
-      frame_provider_->SetFrameData(std::move(frame_data));
     }
+
+    cc::SurfaceFactory::DrawCallback ack_callback =
+        base::Bind(&RenderWidgetHostView::RunAckCallbacks,
+                   weak_ptr_factory_.GetWeakPtr());
+    surface_factory_->SubmitCompositorFrame(surface_id_,
+                                            std::move(frame),
+                                            ack_callback);
   }
 
+  last_submitted_frame_metadata_ = metadata;
   last_frame_size_dip_ = frame_size_dip;
 
   if (layer_.get()) {
     layer_->SetNeedsDisplayRect(damage_rect_dip);
   }
 
-  if (frame_is_evicted_) {
-    frame_is_evicted_ = false;
-    RendererFrameEvictor::GetInstance()->AddFrame(this, is_showing_);
-  }
-
-  compositor_frame_metadata_ = frame->metadata;
-
   bool shrink =
-      compositor_frame_metadata_.location_bar_offset.y() == 0.0f &&
-      compositor_frame_metadata_.location_bar_content_translation.y() > 0.0f;
+      metadata.location_bar_offset.y() == 0.0f &&
+      metadata.location_bar_content_translation.y() > 0.0f;
   if (shrink != top_controls_shrink_blink_size_) {
     top_controls_shrink_blink_size_ = shrink;
     host_->WasResized();
   }
 
-  bool has_mobile_viewport = HasMobileViewport(compositor_frame_metadata_);
-  bool has_fixed_page_scale = HasFixedPageScale(compositor_frame_metadata_);
+  bool has_mobile_viewport = HasMobileViewport(metadata);
+  bool has_fixed_page_scale = HasFixedPageScale(metadata);
   gesture_provider_->SetDoubleTapSupportForPageEnabled(
       !has_fixed_page_scale && !has_mobile_viewport);
 
-  Compositor* compositor = container_ ? container_->GetCompositor() : nullptr;
-  if (!compositor || !compositor->IsActive()) {
-    RunAckCallbacks();
+  if (host_->is_hidden()) {
+    RunAckCallbacks(cc::SurfaceDrawStatus::DRAW_SKIPPED);
   }
 
-  const cc::ViewportSelection& selection = compositor_frame_metadata_.selection;
+  const cc::ViewportSelection& selection = metadata.selection;
   selection_controller_->OnSelectionEditable(selection.is_editable);
   selection_controller_->OnSelectionEmpty(selection.is_empty_text_form_control);
   selection_controller_->OnSelectionBoundsChanged(
@@ -336,9 +377,6 @@ void RenderWidgetHostView::InitAsFullscreen(
   NOTREACHED() << "Fullscreen RenderWidgetHostView's are not supported";
 }
 
-void RenderWidgetHostView::MovePluginWindows(
-    const std::vector<content::WebPluginGeometry>& moves) {}
-
 void RenderWidgetHostView::UpdateCursor(const content::WebCursor& cursor) {
   if (cursor.IsEqual(web_cursor_)) {
     return;
@@ -376,6 +414,8 @@ void RenderWidgetHostView::RenderProcessGone(base::TerminationStatus status,
 }
 
 void RenderWidgetHostView::Destroy() {
+  DestroyDelegatedContent();
+  surface_factory_.reset();
   host_ = nullptr;
   delete this;
 }
@@ -409,8 +449,7 @@ bool RenderWidgetHostView::HasAcceleratedSurface(
 
 void RenderWidgetHostView::GetScreenInfo(blink::WebScreenInfo* result) {
   if (!container_) {
-    *result =
-        BrowserPlatformIntegration::GetInstance()->GetDefaultScreenInfo();
+    RenderWidgetHostViewOxide::GetDefaultScreenInfo(result);
     return;
   }
 
@@ -485,7 +524,7 @@ gfx::Rect RenderWidgetHostView::GetViewBounds() const {
   if (!container_) {
     bounds = gfx::Rect(last_size_);
   } else {
-    bounds = container_->GetViewBoundsDip();
+    bounds = container_->GetViewBounds();
   }
 
   if (DoTopControlsShrinkBlinkSize()) {
@@ -502,7 +541,32 @@ bool RenderWidgetHostView::LockMouse() {
 void RenderWidgetHostView::UnlockMouse() {}
 
 void RenderWidgetHostView::CompositorDidCommit() {
-  RunAckCallbacks();
+  committed_frame_metadata_ = last_submitted_frame_metadata_;
+  RunAckCallbacks(cc::SurfaceDrawStatus::DRAWN);
+}
+
+void RenderWidgetHostView::CompositorWillRequestSwapFrame() {
+  cc::CompositorFrameMetadata old = displayed_frame_metadata_;
+  displayed_frame_metadata_ = committed_frame_metadata_;
+
+  if (!container_) {
+    return;
+  }
+
+  // If the location bar offset changes while a touch selection is active,
+  // the bounding rect and the position of the handles need to be updated.
+  if ((selection_controller_->active_status() !=
+          ui::TouchSelectionController::INACTIVE) &&
+      HasLocationBarOffsetChanged(old, displayed_frame_metadata_)) {
+    NotifyTouchSelectionChanged(false);
+    // XXX: hack to ensure the position of the handles is updated.
+    selection_controller_->SetTemporarilyHidden(true);
+    selection_controller_->SetTemporarilyHidden(false);
+  }
+}
+
+void RenderWidgetHostView::CompositorEvictResources() {
+  DestroyDelegatedContent();
 }
 
 void RenderWidgetHostView::OnGestureEvent(
@@ -538,6 +602,25 @@ void RenderWidgetHostView::OnGestureEvent(
   host_->ForwardGestureEvent(event);
 }
 
+void RenderWidgetHostView::OnUserInput() const {
+  if (selection_controller_->active_status() ==
+          ui::TouchSelectionController::INSERTION_ACTIVE) {
+    selection_controller_->HideAndDisallowShowingAutomatically();
+  }
+}
+
+bool RenderWidgetHostView::HandleContextMenu(
+    const content::ContextMenuParams& params) {
+  if ((params.source_type == ui::MENU_SOURCE_LONG_PRESS) &&
+      params.is_editable &&
+      params.selection_text.empty()) {
+    return true;
+  }
+
+  selection_controller_->HideAndDisallowShowingAutomatically();
+  return false;
+}
+
 bool RenderWidgetHostView::HandleGestureForTouchSelection(
     const blink::WebGestureEvent& event) const {
   switch (event.type) {
@@ -575,17 +658,31 @@ bool RenderWidgetHostView::HandleGestureForTouchSelection(
   return false;
 }
 
-void RenderWidgetHostView::EvictCurrentFrame() {
-  frame_is_evicted_ = true;
-  DestroyDelegatedContent();
+void RenderWidgetHostView::NotifyTouchSelectionChanged(
+    bool insertion_handle_tapped) {
+  if (!container_) {
+    return;
+  }
+
+  container_->TouchSelectionChanged(handle_drag_in_progress_,
+                                    insertion_handle_tapped);
 }
 
-void RenderWidgetHostView::UnusedResourcesAreAvailable() {
+void RenderWidgetHostView::ReturnResources(
+    const cc::ReturnedResourceArray& resources) {
+  if (resources.empty()) {
+    return;
+  }
+  std::copy(resources.begin(), resources.end(),
+            std::back_inserter(surface_returned_resources_));
   if (ack_callbacks_.empty()) {
     SendReturnedDelegatedResources();
   }
 }
 
+void RenderWidgetHostView::SetBeginFrameSource(
+    cc::BeginFrameSource* begin_frame_source) {}
+
 bool RenderWidgetHostView::SupportsAnimation() const {
   return false;
 }
@@ -621,18 +718,32 @@ void RenderWidgetHostView::SelectBetweenCoordinates(const gfx::PointF& base,
 }
 
 void RenderWidgetHostView::OnSelectionEvent(ui::SelectionEventType event) {
-  if (container_) {
-    container_->TouchSelectionChanged();
+  bool insertion_handle_tapped = false;
+  switch (event) {
+    case ui::SELECTION_HANDLE_DRAG_STARTED:
+    case ui::INSERTION_HANDLE_DRAG_STARTED:
+      handle_drag_in_progress_ = true;
+      break;
+    case ui::SELECTION_HANDLE_DRAG_STOPPED:
+    case ui::INSERTION_HANDLE_DRAG_STOPPED:
+      handle_drag_in_progress_ = false;
+      break;
+    case ui::INSERTION_HANDLE_TAPPED:
+      insertion_handle_tapped = true;
+      break;
+    default:
+      break;
   }
+  NotifyTouchSelectionChanged(insertion_handle_tapped);
 }
 
-scoped_ptr<ui::TouchHandleDrawable> RenderWidgetHostView::CreateDrawable() {
+std::unique_ptr<ui::TouchHandleDrawable>
+RenderWidgetHostView::CreateDrawable() {
   if (!container_) {
     return nullptr;
   }
 
-  return scoped_ptr<ui::TouchHandleDrawable>(
-      container_->CreateTouchHandleDrawable());
+  return base::WrapUnique(container_->CreateTouchHandleDrawable());
 }
 
 void RenderWidgetHostView::UpdateCurrentCursor() {
@@ -653,13 +764,17 @@ void RenderWidgetHostView::UpdateCurrentCursor() {
 
 void RenderWidgetHostView::DestroyDelegatedContent() {
   DetachLayer();
-  frame_provider_ = nullptr;
+  if (!surface_id_.is_null()) {
+    DCHECK(surface_factory_.get());
+    surface_factory_->Destroy(surface_id_);
+    surface_id_ = cc::SurfaceId();
+  }
   layer_ = nullptr;
 }
 
 void RenderWidgetHostView::SendDelegatedFrameAck(uint32_t surface_id) {
   cc::CompositorFrameAck ack;
-  resource_collection_->TakeUnusedResourcesForChildCompositor(&ack.resources);
+  ack.resources.swap(surface_returned_resources_);
 
   content::RenderWidgetHostImpl::SendSwapCompositorFrameAck(
       host_->GetRoutingID(),
@@ -669,10 +784,11 @@ void RenderWidgetHostView::SendDelegatedFrameAck(uint32_t surface_id) {
 }
 
 void RenderWidgetHostView::SendReturnedDelegatedResources() {
-  cc::CompositorFrameAck ack;
-  resource_collection_->TakeUnusedResourcesForChildCompositor(&ack.resources);
+  DCHECK(host_);
 
-  DCHECK(!ack.resources.empty());
+  cc::CompositorFrameAck ack;
+  DCHECK(!surface_returned_resources_.empty());
+  ack.resources.swap(surface_returned_resources_);
 
   content::RenderWidgetHostImpl::SendReclaimCompositorResources(
       host_->GetRoutingID(),
@@ -681,7 +797,7 @@ void RenderWidgetHostView::SendReturnedDelegatedResources() {
       ack);
 }
 
-void RenderWidgetHostView::RunAckCallbacks() {
+void RenderWidgetHostView::RunAckCallbacks(cc::SurfaceDrawStatus status) {
   while (!ack_callbacks_.empty()) {
     ack_callbacks_.front().Run();
     ack_callbacks_.pop();
@@ -714,17 +830,18 @@ RenderWidgetHostView::RenderWidgetHostView(
     content::RenderWidgetHostImpl* host)
     : host_(host),
       container_(nullptr),
-      resource_collection_(new cc::DelegatedFrameResourceCollection()),
+      id_allocator_(
+          CompositorUtils::GetInstance()->CreateSurfaceIdAllocator()),
       last_output_surface_id_(0),
-      frame_is_evicted_(true),
       ime_bridge_(this),
       is_loading_(false),
       is_showing_(!host->is_hidden()),
       top_controls_shrink_blink_size_(false),
-      gesture_provider_(GestureProvider::Create(this)) {
+      gesture_provider_(GestureProvider::Create(this)),
+      handle_drag_in_progress_(false),
+      weak_ptr_factory_(this) {
   CHECK(host_) << "Implementation didn't supply a RenderWidgetHost";
 
-  resource_collection_->SetClient(this);
   host_->SetView(this);
 
   gesture_provider_->SetDoubleTapSupportForPageEnabled(false);
@@ -733,16 +850,16 @@ RenderWidgetHostView::RenderWidgetHostView(
   // default values from ui/events/gesture_detection/gesture_configuration.cc
   tsc_config.max_tap_duration = base::TimeDelta::FromMilliseconds(150);
   tsc_config.tap_slop = 15;
-  tsc_config.enable_adaptive_handle_orientation = false;
-  tsc_config.show_on_tap_for_empty_editable = true;
+  tsc_config.enable_adaptive_handle_orientation = true;
   tsc_config.enable_longpress_drag_selection = false;
   selection_controller_.reset(
       new ui::TouchSelectionController(this, tsc_config));
 }
 
 RenderWidgetHostView::~RenderWidgetHostView() {
-  resource_collection_->SetClient(nullptr);
-  SetContainer(nullptr);
+  DCHECK(!layer_);
+  DCHECK(!surface_factory_);
+  DCHECK(surface_id_.is_null());
 }
 
 void RenderWidgetHostView::SetContainer(
@@ -789,7 +906,7 @@ void RenderWidgetHostView::HandleTouchEvent(const ui::MotionEvent& event) {
   }
 
   host_->ForwardTouchEventWithLatencyInfo(
-      MakeWebTouchEvent(event, rv.did_generate_scroll),
+      MakeWebTouchEvent(event, rv.moved_beyond_slop_region),
       ui::LatencyInfo());
 }
 
@@ -797,7 +914,7 @@ void RenderWidgetHostView::ResetGestureDetection() {
   const ui::MotionEvent* current_down_event =
       gesture_provider_->GetCurrentDownEvent();
   if (current_down_event) {
-    scoped_ptr<ui::MotionEvent> cancel_event = current_down_event->Cancel();
+    std::unique_ptr<ui::MotionEvent> cancel_event = current_down_event->Cancel();
     HandleTouchEvent(*cancel_event);
   }
 
@@ -831,10 +948,6 @@ void RenderWidgetHostView::Show() {
     layer_->SetHideLayerAndSubtree(false);
   }
 
-  if (!frame_is_evicted_) {
-    RendererFrameEvictor::GetInstance()->LockFrame(this);
-  }
-
   if (!host_ || !host_->is_hidden()) {
     return;
   }
@@ -852,11 +965,7 @@ void RenderWidgetHostView::Hide() {
     layer_->SetHideLayerAndSubtree(true);
   }
 
-  if (!frame_is_evicted_) {
-    RendererFrameEvictor::GetInstance()->UnlockFrame(this);
-  }
-
-  RunAckCallbacks();
+  RunAckCallbacks(cc::SurfaceDrawStatus::DRAW_SKIPPED);
 
   if (!host_ || host_->is_hidden()) {
     return;
