@@ -18,13 +18,11 @@
 #ifndef _OXIDE_SHARED_BROWSER_WEB_VIEW_H_
 #define _OXIDE_SHARED_BROWSER_WEB_VIEW_H_
 
-#include <queue>
+#include <memory>
 #include <string>
 #include <vector>
 
 #include "base/macros.h"
-#include "base/memory/ref_counted.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/strings/string16.h"
 #include "cc/output/compositor_frame_metadata.h"
@@ -37,70 +35,48 @@
 #include "content/public/common/javascript_message_type.h"
 #include "third_party/WebKit/public/platform/WebScreenInfo.h"
 #include "third_party/WebKit/public/platform/WebTopControlsState.h"
+#include "third_party/WebKit/public/web/WebContextMenuData.h"
 #include "ui/gfx/geometry/point.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/size.h"
 
-#include "shared/browser/compositor/oxide_compositor_client.h"
 #include "shared/browser/compositor/oxide_compositor_observer.h"
-#include "shared/browser/input/oxide_input_method_context_observer.h"
 #include "shared/browser/oxide_browser_platform_integration_observer.h"
 #include "shared/browser/oxide_content_types.h"
 #include "shared/browser/oxide_render_object_id.h"
-#include "shared/browser/oxide_render_widget_host_view_container.h"
 #include "shared/browser/oxide_script_message_target.h"
-#include "shared/browser/oxide_security_status.h"
-#include "shared/browser/oxide_security_types.h"
-#include "shared/browser/oxide_touch_event_state.h"
 #include "shared/browser/oxide_web_preferences_observer.h"
+#include "shared/browser/ssl/oxide_security_status.h"
+#include "shared/browser/ssl/oxide_security_types.h"
 #include "shared/common/oxide_message_enums.h"
+#include "shared/common/oxide_shared_export.h"
 
 class GURL;
 
-namespace blink {
-class WebMouseEvent;
-class WebMouseWheelEvent;
-} // namespace blink
-
-namespace cc {
-class SolidColorLayer;
-}
-
 namespace content {
 
-class NativeWebKeyboardEvent;
 class NotificationRegistrar;
 struct OpenURLParams;
 class RenderFrameHost;
 class RenderViewHost;
 class RenderWidgetHost;
 class WebContents;
-class WebContentsImpl;
 
 } // namespace content
 
 namespace gfx {
 class Range;
-class Size;
-}
-
-namespace ui {
-class TouchEvent;
-class TouchHandleDrawable;
 }
 
 namespace oxide {
 
 class BrowserContext;
-class Compositor;
-class CompositorFrameHandle;
 class FilePicker;
 class JavaScriptDialog;
 class ResourceDispatcherHostLoginDelegate;
 class RenderWidgetHostView;
-class WebContextMenu;
+class WebContentsViewClient;
 class WebFrame;
-class WebPopupMenu;
 class WebPreferences;
 class WebView;
 class WebViewClient;
@@ -125,33 +101,40 @@ class WebViewIterator final {
 
 // This is the main webview class. Implementations should customize this by
 // providing an implementation of WebViewClient
-class WebView : public ScriptMessageTarget,
-                private InputMethodContextObserver,
-                private CompositorObserver,
-                private CompositorClient,
-                private WebPreferencesObserver,
-                private content::NotificationObserver,
-                private RenderWidgetHostViewContainer,
-                private content::WebContentsDelegate,
-                private content::WebContentsObserver,
-                private BrowserPlatformIntegrationObserver {
+class OXIDE_SHARED_EXPORT WebView : public ScriptMessageTarget,
+                                    private CompositorObserver,
+                                    private WebPreferencesObserver,
+                                    private content::NotificationObserver,
+                                    private content::WebContentsDelegate,
+                                    private content::WebContentsObserver,
+                                    private BrowserPlatformIntegrationObserver {
  public:
 
-  struct Params {
-    Params();
-    ~Params();
+  struct CommonParams {
+    CommonParams();
+    ~CommonParams();
 
     WebViewClient* client;
+    WebContentsViewClient* view_client;
+  };
+
+  struct CreateParams {
+    CreateParams();
+    ~CreateParams();
+
     BrowserContext* context;
     bool incognito;
     std::vector<sessions::SerializedNavigationEntry> restore_entries;
     content::NavigationController::RestoreType restore_type;
     int restore_index;
+    gfx::Size initial_size;
+    bool initially_hidden;
   };
 
-  WebView(const Params& params);
-  WebView(scoped_ptr<content::WebContents> contents,
-          WebViewClient* client);
+  WebView(const CommonParams& common_params,
+          const CreateParams& create_params);
+  WebView(const CommonParams& common_params,
+          std::unique_ptr<content::WebContents> contents);
 
   ~WebView() override;
 
@@ -192,13 +175,6 @@ class WebView : public ScriptMessageTarget,
 
   bool IsLoading() const;
 
-  bool FullscreenGranted() const;
-  void SetFullscreenGranted(bool fullscreen);
-
-  void WasResized();
-  void ScreenUpdated();
-  void VisibilityChanged();
-  void FocusChanged();
   void UpdateWebPreferences();
 
   BrowserContext* GetBrowserContext() const;
@@ -217,28 +193,23 @@ class WebView : public ScriptMessageTarget,
   WebPreferences* GetWebPreferences();
   void SetWebPreferences(WebPreferences* prefs);
 
-  gfx::Size GetViewSizePix() const;
-  gfx::Rect GetViewBoundsDip() const;
-  gfx::Size GetViewSizeDip() const;
-
   const cc::CompositorFrameMetadata& compositor_frame_metadata() const {
     return compositor_frame_metadata_;
   }
-  gfx::Point GetCompositorFrameScrollOffsetPix();
-  gfx::Size GetCompositorFrameContentSizePix();
-  gfx::Size GetCompositorFrameViewportSizePix();
 
-  int GetLocationBarOffsetPix() const;
-  int GetLocationBarContentOffsetPix() const;
-  float GetLocationBarContentOffsetDip() const;
+  gfx::Point GetCompositorFrameScrollOffset();
+  gfx::Size GetCompositorFrameContentSize();
+  gfx::Size GetCompositorFrameViewportSize();
+
+  float GetLocationBarOffset() const;
+  float GetLocationBarContentOffset() const;
 
   const SecurityStatus& security_status() const { return security_status_; }
 
   ContentType blocked_content() const { return blocked_content_; }
 
-  float GetLocationBarHeightDip() const;
-  int GetLocationBarHeightPix() const;
-  void SetLocationBarHeightPix(int height);
+  float GetLocationBarHeight() const;
+  void SetLocationBarHeight(float height);
 
   blink::WebTopControlsState location_bar_constraints() const {
     return location_bar_constraints_;
@@ -258,11 +229,6 @@ class WebView : public ScriptMessageTarget,
 
   void PrepareToClose();
 
-  void HandleKeyEvent(const content::NativeWebKeyboardEvent& event);
-  void HandleMouseEvent(const blink::WebMouseEvent& event);
-  void HandleTouchEvent(const ui::TouchEvent& event);
-  void HandleWheelEvent(const blink::WebMouseWheelEvent& event);
-
   void DownloadRequested(
       const GURL& url,
       const std::string& mime_type,
@@ -275,13 +241,7 @@ class WebView : public ScriptMessageTarget,
   void HttpAuthenticationRequested(
       ResourceDispatcherHostLoginDelegate* login_delegate);
 
-  CompositorFrameHandle* GetCompositorFrameHandle() const;
-  void DidCommitCompositorFrame();
-
   blink::WebScreenInfo GetScreenInfo() const;
-  gfx::Rect GetViewBoundsPix() const;
-  bool IsVisible() const;
-  bool HasFocus() const;
 
   JavaScriptDialog* CreateJavaScriptDialog(
       content::JavaScriptMessageType javascript_message_type);
@@ -293,17 +253,23 @@ class WebView : public ScriptMessageTarget,
 
   const GURL& target_url() const { return target_url_; }
 
-  int GetEditFlags() const;
+  blink::WebContextMenuData::EditFlags GetEditFlags() const;
 
  private:
   WebView(WebViewClient* client);
 
-  void CommonInit(scoped_ptr<content::WebContents> contents);
+  void CommonInit(std::unique_ptr<content::WebContents> contents,
+                  WebContentsViewClient* view_client);
 
   RenderWidgetHostView* GetRenderWidgetHostView() const;
   content::RenderViewHost* GetRenderViewHost() const;
   content::RenderWidgetHost* GetRenderWidgetHost() const;
 
+  gfx::Size GetViewSizeDip() const;
+  gfx::Rect GetViewBoundsDip() const;
+
+  bool IsFullscreen() const;
+
   void DispatchLoadFailed(const GURL& validated_url,
                           int error_code,
                           const base::string16& error_description,
@@ -312,67 +278,39 @@ class WebView : public ScriptMessageTarget,
   void OnDidBlockDisplayingInsecureContent();
   void OnDidBlockRunningInsecureContent();
 
-  bool ShouldScrollFocusedEditableNodeIntoView();
-  void MaybeScrollFocusedEditableNodeIntoView();
-
-  float GetFrameMetadataScaleToPix();
-
   void InitializeTopControlsForHost(content::RenderViewHost* rvh,
                                     bool initial_host);
 
   void DispatchPrepareToCloseResponse(bool proceed);
 
-  void RestartFindInPage();
-
   void MaybeCancelFullscreenMode();
 
+  void EditingCapabilitiesChanged();
+
   // ScriptMessageTarget implementation
   virtual size_t GetScriptMessageHandlerCount() const override;
   virtual const ScriptMessageHandler* GetScriptMessageHandlerAt(
       size_t index) const override;
 
-  // InputMethodContextObserver implementation
-  void InputPanelVisibilityChanged() override;
 
   // CompositorObserver implementation
-  void CompositorDidCommit() final;
-
-  // CompositorClient implementation
-  void CompositorSwapFrame(CompositorFrameHandle* handle) final;
+  void CompositorWillRequestSwapFrame() override;
 
   // WebPreferencesObserver implementation
-  void WebPreferencesDestroyed() final;
+  void WebPreferencesDestroyed() override;
 
   // content::NotificationObserver implementation
   void Observe(int type,
                const content::NotificationSource& source,
-               const content::NotificationDetails& details) final;
-
-  // RenderWidgetHostViewContainer implementation
-  Compositor* GetCompositor() const final;
-  void AttachLayer(scoped_refptr<cc::Layer> layer) final;
-  void DetachLayer(scoped_refptr<cc::Layer> layer) final;
-  void CursorChanged() final;
-  bool HasFocus(const RenderWidgetHostView* view) const final;
-  bool IsFullscreen() const final;
-  void ShowContextMenu(content::RenderFrameHost* render_frame_host,
-                       const content::ContextMenuParams& params) final;
-  void ShowPopupMenu(content::RenderFrameHost* render_frame_host,
-                     const gfx::Rect& bounds,
-                     int selected_item,
-                     const std::vector<content::MenuItem>& items,
-                     bool allow_multiple_selection) final;
-  void HidePopupMenu() final;
-  ui::TouchHandleDrawable* CreateTouchHandleDrawable() const final;
-  void TouchSelectionChanged() const final;
-  void EditingCapabilitiesChanged() final;
+               const content::NotificationDetails& details) override;
 
   // content::WebContentsDelegate implementation
-  content::WebContents* OpenURLFromTab(content::WebContents* source,
-                                       const content::OpenURLParams& params) final;
+  content::WebContents* OpenURLFromTab(
+      content::WebContents* source,
+      const content::OpenURLParams& params) override;
   void NavigationStateChanged(content::WebContents* source,
-                              content::InvalidateTypes changed_flags) final;
-  void VisibleSSLStateChanged(const content::WebContents* source) final;
+                              content::InvalidateTypes changed_flags) override;
+  void VisibleSSLStateChanged(const content::WebContents* source) override;
   bool ShouldCreateWebContents(
       content::WebContents* source,
       int route_id,
@@ -384,150 +322,132 @@ class WebView : public ScriptMessageTarget,
       const std::string& partition_id,
       content::SessionStorageNamespace* session_storage_namespace,
       WindowOpenDisposition disposition,
-      bool user_gesture) final;
-  void HandleKeyboardEvent(content::WebContents* source,
-                           const content::NativeWebKeyboardEvent& event) final;
+      bool user_gesture) override;
+  void HandleKeyboardEvent(
+      content::WebContents* source,
+      const content::NativeWebKeyboardEvent& event) override;
   void WebContentsCreated(content::WebContents* source,
                           int source_frame_id,
                           const std::string& frame_name,
                           const GURL& target_url,
-                          content::WebContents* new_contents) final;
+                          content::WebContents* new_contents) override;
   void AddNewContents(content::WebContents* source,
                       content::WebContents* new_contents,
                       WindowOpenDisposition disposition,
                       const gfx::Rect& initial_pos,
                       bool user_gesture,
-                      bool* was_blocked) final;
-  void LoadProgressChanged(content::WebContents* source, double progress) final;
-  void CloseContents(content::WebContents* source) final;
-  void UpdateTargetURL(content::WebContents* source, const GURL& url) final;
+                      bool* was_blocked) override;
+  void LoadProgressChanged(content::WebContents* source,
+                           double progress) override;
+  void CloseContents(content::WebContents* source) override;
+  void UpdateTargetURL(content::WebContents* source, const GURL& url) override;
   bool AddMessageToConsole(content::WebContents* source,
                int32_t level,
                const base::string16& message,
                int32_t line_no,
-               const base::string16& source_id) final;
+               const base::string16& source_id) override;
   void BeforeUnloadFired(content::WebContents* source,
                          bool proceed,
-                         bool* proceed_to_fire_unload) final;
+                         bool* proceed_to_fire_unload) override;
   content::JavaScriptDialogManager* GetJavaScriptDialogManager(
-      content::WebContents* source) final;
+      content::WebContents* source) override;
   void RunFileChooser(content::WebContents* web_contents,
-                      const content::FileChooserParams& params) final;
-  bool EmbedsFullscreenWidget() const final;
+                      const content::FileChooserParams& params) override;
+  bool EmbedsFullscreenWidget() const override;
   void EnterFullscreenModeForTab(content::WebContents* source,
-                                 const GURL& origin) final;
-  void ExitFullscreenModeForTab(content::WebContents* source) final;
+                                 const GURL& origin) override;
+  void ExitFullscreenModeForTab(content::WebContents* source) override;
   bool IsFullscreenForTabOrPending(
-      const content::WebContents* source) const final;
+      const content::WebContents* source) const override;
   void FindReply(content::WebContents* source,
                  int request_id,
                  int number_of_matches,
                  const gfx::Rect& selection_rect,
                  int active_match_ordinal,
-                 bool final_update) final;
+                 bool final_update) override;
   void RequestMediaAccessPermission(
       content::WebContents* source,
       const content::MediaStreamRequest& request,
-      const content::MediaResponseCallback& callback) final;
+      const content::MediaResponseCallback& callback) override;
   bool CheckMediaAccessPermission(content::WebContents* source,
                                   const GURL& security_origin,
-                                  content::MediaStreamType type) final;
+                                  content::MediaStreamType type) override;
 
   // content::WebContentsObserver implementation
   void RenderFrameForInterstitialPageCreated(
-      content::RenderFrameHost* render_frame_host) final;
-  void RenderViewReady() final;
-  void RenderProcessGone(base::TerminationStatus status) final;
+      content::RenderFrameHost* render_frame_host) override;
+  void RenderViewReady() override;
+  void RenderProcessGone(base::TerminationStatus status) override;
   void RenderViewHostChanged(content::RenderViewHost* old_host,
-                             content::RenderViewHost* new_host) final;
-  void DidStartLoading() final;
-  void DidStopLoading() final;
+                             content::RenderViewHost* new_host) override;
+  void DidStartLoading() override;
+  void DidStopLoading() override;
   void DidFinishLoad(content::RenderFrameHost* render_frame_host,
-                     const GURL& validated_url) final;
+                     const GURL& validated_url) override;
   void DidFailLoad(content::RenderFrameHost* render_frame_host,
                    const GURL& validated_url,
                    int error_code,
                    const base::string16& error_description,
-                   bool was_ignored_by_handler) final;
+                   bool was_ignored_by_handler) override;
   void DidStartProvisionalLoadForFrame(
       content::RenderFrameHost* render_frame_host,
       const GURL& validated_url,
       bool is_error_page,
-      bool is_iframe_srcdoc) final;
+      bool is_iframe_srcdoc) override;
   void DidCommitProvisionalLoadForFrame(
       content::RenderFrameHost* render_frame_host,
       const GURL& url,
-      ui::PageTransition transition_type) final;
+      ui::PageTransition transition_type) override;
   void DidFailProvisionalLoad(
       content::RenderFrameHost* render_frame_host,
       const GURL& validated_url,
       int error_code,
       const base::string16& error_description,
-      bool was_ignored_by_handler) final;
+      bool was_ignored_by_handler) override;
   void DidNavigateMainFrame(
       const content::LoadCommittedDetails& details,
-      const content::FrameNavigateParams& params) final;
+      const content::FrameNavigateParams& params) override;
   void DidGetRedirectForResourceRequest(
       content::RenderFrameHost* render_frame_host,
-      const content::ResourceRedirectDetails& details) final;
+      const content::ResourceRedirectDetails& details) override;
   void NavigationEntryCommitted(
-      const content::LoadCommittedDetails& load_details) final;
-  void TitleWasSet(content::NavigationEntry* entry, bool explicit_set) final;
-  void DidShowFullscreenWidget(int routing_id) final;
-  void DidDestroyFullscreenWidget(int routing_id) final;
-  void DidAttachInterstitialPage() final;
-  void DidDetachInterstitialPage() final;
+      const content::LoadCommittedDetails& load_details) override;
+  void TitleWasSet(content::NavigationEntry* entry, bool explicit_set) override;
+  void DidShowFullscreenWidget(int routing_id) override;
   bool OnMessageReceived(const IPC::Message& msg,
-                         content::RenderFrameHost* render_frame_host) final;
+                         content::RenderFrameHost* render_frame_host) override;
 
   // BrowserPlatformIntegrationObserver implementation
-  void ClipboardDataChanged() final;
+  void ClipboardDataChanged() override;
 
   WebViewClient* client_;
 
   struct WebContentsDeleter {
     void operator()(content::WebContents* contents);
   };
-  typedef scoped_ptr<content::WebContents, WebContentsDeleter>
+  typedef std::unique_ptr<content::WebContents, WebContentsDeleter>
       WebContentsScopedPtr;
 
   WebContentsScopedPtr web_contents_;
   WebViewContentsHelper* web_contents_helper_;
 
-  scoped_ptr<Compositor> compositor_;
-  scoped_refptr<cc::SolidColorLayer> root_layer_;
-
-  scoped_refptr<CompositorFrameHandle> current_compositor_frame_;
-  std::vector<scoped_refptr<CompositorFrameHandle> > previous_compositor_frames_;
-  std::queue<uint32_t> received_surface_ids_;
-
-  gfx::Point global_mouse_position_;
-  TouchEventState touch_state_;
-
   content::NotificationRegistrar registrar_;
 
-  bool fullscreen_granted_;
-  bool fullscreen_requested_;
-
-  base::WeakPtr<WebPopupMenu> active_popup_menu_;
   base::WeakPtr<FilePicker> active_file_picker_;
 
   ContentType blocked_content_;
 
-  cc::CompositorFrameMetadata pending_compositor_frame_metadata_;
   cc::CompositorFrameMetadata compositor_frame_metadata_;
 
   SecurityStatus security_status_;
 
-  int location_bar_height_pix_;
+  int location_bar_height_;
   blink::WebTopControlsState location_bar_constraints_;
   bool location_bar_animated_;
 
-  RenderWidgetHostID interstitial_rwh_id_;
-
   GURL target_url_;
 
-  int edit_flags_;
+  blink::WebContextMenuData::EditFlags edit_flags_;
 
   base::WeakPtrFactory<WebView> weak_factory_;
 
