@@ -5,20 +5,20 @@
 #ifndef _OXIDE_RENDERER_MEDIA_WEBMEDIAPLAYER_OXIDE_H_
 #define _OXIDE_RENDERER_MEDIA_WEBMEDIAPLAYER_OXIDE_H_
 
+#include <memory>
 #include <string>
 #include <vector>
 
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/time/time.h"
 #include "base/threading/thread_checker.h"
 #include "content/public/renderer/render_frame_observer.h"
-#include "media/cdm/proxy_decryptor.h"
 #include "gpu/command_buffer/common/mailbox.h"
 #include "media/base/demuxer_stream.h"
 #include "media/base/media_keys.h"
+#include "media/blink/webmediaplayer_delegate.h"
 #include "third_party/WebKit/public/platform/WebGraphicsContext3D.h"
 #include "third_party/WebKit/public/platform/WebMediaPlayer.h"
 #include "third_party/WebKit/public/platform/WebSize.h"
@@ -45,7 +45,6 @@ struct MailboxHolder;
 
 namespace media {
 class MediaLog;
-class WebMediaPlayerDelegate;
 }
 
 namespace oxide {
@@ -53,7 +52,8 @@ class RendererMediaPlayerManager;
 class WebContentDecryptionModuleImpl;
 
 class WebMediaPlayer : public blink::WebMediaPlayer,
-                       public content::RenderFrameObserver {
+                       public content::RenderFrameObserver,
+                       public NON_EXPORTED_BASE(media::WebMediaPlayerDelegate::Observer){
  public:
   WebMediaPlayer(blink::WebFrame* frame,
                         blink::WebMediaPlayerClient* client,
@@ -73,8 +73,8 @@ class WebMediaPlayer : public blink::WebMediaPlayer,
 
   // Resource loading.
   void load(LoadType load_type,
-                    const blink::WebURL& url,
-                    CORSMode cors_mode);
+            const blink::WebURL& url,
+            CORSMode cors_mode);
 
   // Playback controls.
   void play();
@@ -118,11 +118,12 @@ class WebMediaPlayer : public blink::WebMediaPlayer,
   double timelineOffset() const;
   double currentTime() const;
 
+  blink::WebString getErrorMessage();
   bool didLoadingProgress();
 
   // Internal states of loading and network.
-  blink::WebMediaPlayer::NetworkState networkState() const;
-  blink::WebMediaPlayer::ReadyState readyState() const;
+  blink::WebMediaPlayer::NetworkState getNetworkState() const;
+  blink::WebMediaPlayer::ReadyState getReadyState() const;
 
   bool hasSingleSecurityOrigin() const;
   bool didPassCORSAccessCheck() const;
@@ -132,8 +133,8 @@ class WebMediaPlayer : public blink::WebMediaPlayer,
   // Provide statistics.
   unsigned decodedFrameCount() const;
   unsigned droppedFrameCount() const;
-  unsigned audioDecodedByteCount() const;
-  unsigned videoDecodedByteCount() const;
+  size_t audioDecodedByteCount() const;
+  size_t videoDecodedByteCount() const;
 
   void paint(blink::WebCanvas*, const blink::WebRect&, unsigned char alpha, SkXfermode::Mode);
     //
@@ -172,22 +173,13 @@ class WebMediaPlayer : public blink::WebMediaPlayer,
   // Detach the player from its manager.
   void Detach();
 
-  MediaKeyException generateKeyRequest(
-      const blink::WebString& key_system,
-      const unsigned char* init_data,
-      unsigned init_data_length);
-  MediaKeyException addKey(
-      const blink::WebString& key_system,
-      const unsigned char* key,
-      unsigned key_length,
-      const unsigned char* init_data,
-      unsigned init_data_length,
-      const blink::WebString& session_id);
-  MediaKeyException cancelKeyRequest(
-      const blink::WebString& key_system,
-      const blink::WebString& session_id);
-  void setContentDecryptionModule(
-      blink::WebContentDecryptionModule* cdm);
+  // WebMediaPlayerDelegate::Observer implementation.
+  void OnHidden() override;
+  void OnShown() override;
+  void OnSuspendRequested(bool must_suspend) override;
+  void OnPlay() override;
+  void OnPause() override;
+  void OnVolumeMultiplierUpdate(double multiplier) override;
 
  protected:
   // Helper method to update the playing state.
@@ -200,27 +192,17 @@ class WebMediaPlayer : public blink::WebMediaPlayer,
  private:
   void Pause(bool is_media_related_action);
   void DidLoadMediaInfo(MediaInfoLoader::Status status);
-  bool IsKeySystemSupported(const std::string& key_system);
-
-  // Actually do the work for generateKeyRequest/addKey so they can easily
-  // report results to UMA.
-  MediaKeyException GenerateKeyRequestInternal(const std::string& key_system,
-                                               const unsigned char* init_data,
-                                               unsigned init_data_length);
-  MediaKeyException AddKeyInternal(const std::string& key_system,
-                                   const unsigned char* key,
-                                   unsigned key_length,
-                                   const unsigned char* init_data,
-                                   unsigned init_data_length,
-                                   const std::string& session_id);
-  MediaKeyException CancelKeyRequestInternal(const std::string& key_system,
-                                             const std::string& session_id);
 
   blink::WebFrame* const frame_;
 
   blink::WebMediaPlayerClient* const client_;
 
+  // WebMediaPlayer notifies the |delegate_| of playback state changes using
+  // |delegate_id_|; an id provided after registering with the delegate.  The
+  // WebMediaPlayer may also receive directives (play, pause) from the delegate
+  // via the WebMediaPlayerDelegate::Observer interface after registration.
   base::WeakPtr<media::WebMediaPlayerDelegate> delegate_;
+  int delegate_id_;
 
   // Save the list of buffered time ranges.
   blink::WebTimeRanges buffered_;
@@ -296,16 +278,12 @@ class WebMediaPlayer : public blink::WebMediaPlayer,
 
   media::MediaLog* media_log_;
 
-  scoped_ptr<MediaInfoLoader> info_loader_;
+  std::unique_ptr<MediaInfoLoader> info_loader_;
 
   // The currently selected key system. Empty string means that no key system
   // has been selected.
   std::string current_key_system_;
 
-  // Temporary for EME v0.1. In the future the init data type should be passed
-  // through GenerateKeyRequest() directly from WebKit.
-  std::string init_data_type_;
-
   // NOTE: Weak pointers must be invalidated before all other member variables.
   base::WeakPtrFactory<WebMediaPlayer> weak_factory_;
 
