@@ -17,11 +17,12 @@
 
 #include "oxide_script_message_request_impl_renderer.h"
 
+#include <memory>
+
 #include "base/logging.h"
 #include "content/public/child/v8_value_converter.h"
 #include "content/public/renderer/render_frame.h"
 #include "third_party/WebKit/public/web/WebLocalFrame.h"
-#include "third_party/WebKit/public/web/WebScopedMicrotaskSuppression.h"
 
 #include "shared/common/oxide_messages.h"
 
@@ -41,7 +42,8 @@ void ScriptMessageRequestImplRenderer::DispatchResponse(
     v8::Local<v8::Value> argv[]) {
   v8::TryCatch try_catch;
   {
-    blink::WebScopedMicrotaskSuppression mts;
+    v8::MicrotasksScope microtasks(manager()->isolate(),
+                                   v8::MicrotasksScope::kDoNotRunMicrotasks);
     manager()->frame()->GetWebFrame()->callFunctionEvenIfScriptDisabled(
         function, GetHandle(), argc, argv);
   }
@@ -60,7 +62,7 @@ void ScriptMessageRequestImplRenderer::OnReply(const base::Value& payload) {
     return;
   }
 
-  scoped_ptr<content::V8ValueConverter> converter(
+  std::unique_ptr<content::V8ValueConverter> converter(
       content::V8ValueConverter::create());
 
   v8::Local<v8::Value> argv[] = {
@@ -82,7 +84,7 @@ void ScriptMessageRequestImplRenderer::OnError(
     return;
   }
 
-  scoped_ptr<content::V8ValueConverter> converter(
+  std::unique_ptr<content::V8ValueConverter> converter(
       content::V8ValueConverter::create());
 
   v8::Local<v8::Value> argv[] = {
